# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: perp/v1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from perp.v1 import state_pb2 as perp_dot_v1_dot_state__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x10perp/v1/tx.proto\x12\x0enibiru.perp.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x14gogoproto/gogo.proto\x1a\x13perp/v1/state.proto\"f\n\x0fMsgRemoveMargin\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x12\n\ntoken_pair\x18\x02 \x01(\t\x12/\n\x06margin\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"\x97\x01\n\x17MsgRemoveMarginResponse\x12\x33\n\nmargin_out\x18\x01 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\x12G\n\x0f\x66unding_payment\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"c\n\x0cMsgAddMargin\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x12\n\ntoken_pair\x18\x02 \x01(\t\x12/\n\x06margin\x18\x03 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"\x8b\x01\n\x14MsgAddMarginResponse\x12G\n\x0f\x66unding_payment\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12*\n\x08position\x18\x02 \x01(\x0b\x32\x18.nibiru.perp.v1.Position\"B\n\x0cMsgLiquidate\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x12\n\ntoken_pair\x18\x02 \x01(\t\x12\x0e\n\x06trader\x18\x03 \x01(\t\"\x97\x01\n\x14MsgLiquidateResponse\x12:\n\x11\x66\x65\x65_to_liquidator\x18\x01 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\x12\x43\n\x1a\x66\x65\x65_to_perp_ecosystem_fund\x18\x02 \x01(\x0b\x32\x19.cosmos.base.v1beta1.CoinB\x04\xc8\xde\x1f\x00\"\xb8\x02\n\x0fMsgOpenPosition\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x12\n\ntoken_pair\x18\x02 \x01(\t\x12\"\n\x04side\x18\x03 \x01(\x0e\x32\x14.nibiru.perp.v1.Side\x12J\n\x12quote_asset_amount\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\xc8\xde\x1f\x00\x12@\n\x08leverage\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12O\n\x17\x62\x61se_asset_amount_limit\x18\x06 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Int\xc8\xde\x1f\x00\"\xd9\x04\n\x17MsgOpenPositionResponse\x12*\n\x08position\x18\x01 \x01(\x0b\x32\x18.nibiru.perp.v1.Position\x12P\n\x18\x65xchanged_notional_value\x18\x02 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12O\n\x17\x65xchanged_position_size\x18\x03 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12G\n\x0f\x66unding_payment\x18\x04 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12\x44\n\x0crealized_pnl\x18\x05 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12L\n\x14unrealized_pnl_after\x18\x06 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12G\n\x0fmargin_to_vault\x18\x07 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\x12I\n\x11position_notional\x18\x08 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"6\n\x10MsgClosePosition\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x12\n\ntoken_pair\x18\x02 \x01(\t\"\x1a\n\x18MsgClosePositionResponse2\xe7\x04\n\x03Msg\x12|\n\x0cRemoveMargin\x12\x1f.nibiru.perp.v1.MsgRemoveMargin\x1a\'.nibiru.perp.v1.MsgRemoveMarginResponse\"\"\x82\xd3\xe4\x93\x02\x1c\"\x1a/nibiru/perp/remove_margin\x12p\n\tAddMargin\x12\x1c.nibiru.perp.v1.MsgAddMargin\x1a$.nibiru.perp.v1.MsgAddMarginResponse\"\x1f\x82\xd3\xe4\x93\x02\x19\"\x17/nibiru/perp/add_margin\x12o\n\tLiquidate\x12\x1c.nibiru.perp.v1.MsgLiquidate\x1a$.nibiru.perp.v1.MsgLiquidateResponse\"\x1e\x82\xd3\xe4\x93\x02\x18\"\x16/nibiru/perp/liquidate\x12|\n\x0cOpenPosition\x12\x1f.nibiru.perp.v1.MsgOpenPosition\x1a\'.nibiru.perp.v1.MsgOpenPositionResponse\"\"\x82\xd3\xe4\x93\x02\x1c\"\x1a/nibiru/perp/open_position\x12\x80\x01\n\rClosePosition\x12 .nibiru.perp.v1.MsgClosePosition\x1a(.nibiru.perp.v1.MsgClosePositionResponse\"#\x82\xd3\xe4\x93\x02\x1d\"\x1b/nibiru/perp/close_positionB,Z*github.com/NibiruChain/nibiru/x/perp/typesb\x06proto3'
)


_MSGREMOVEMARGIN = DESCRIPTOR.message_types_by_name['MsgRemoveMargin']
_MSGREMOVEMARGINRESPONSE = DESCRIPTOR.message_types_by_name['MsgRemoveMarginResponse']
_MSGADDMARGIN = DESCRIPTOR.message_types_by_name['MsgAddMargin']
_MSGADDMARGINRESPONSE = DESCRIPTOR.message_types_by_name['MsgAddMarginResponse']
_MSGLIQUIDATE = DESCRIPTOR.message_types_by_name['MsgLiquidate']
_MSGLIQUIDATERESPONSE = DESCRIPTOR.message_types_by_name['MsgLiquidateResponse']
_MSGOPENPOSITION = DESCRIPTOR.message_types_by_name['MsgOpenPosition']
_MSGOPENPOSITIONRESPONSE = DESCRIPTOR.message_types_by_name['MsgOpenPositionResponse']
_MSGCLOSEPOSITION = DESCRIPTOR.message_types_by_name['MsgClosePosition']
_MSGCLOSEPOSITIONRESPONSE = DESCRIPTOR.message_types_by_name['MsgClosePositionResponse']
MsgRemoveMargin = _reflection.GeneratedProtocolMessageType(
    'MsgRemoveMargin',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGREMOVEMARGIN,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgRemoveMargin)
    },
)
_sym_db.RegisterMessage(MsgRemoveMargin)

MsgRemoveMarginResponse = _reflection.GeneratedProtocolMessageType(
    'MsgRemoveMarginResponse',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGREMOVEMARGINRESPONSE,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgRemoveMarginResponse)
    },
)
_sym_db.RegisterMessage(MsgRemoveMarginResponse)

MsgAddMargin = _reflection.GeneratedProtocolMessageType(
    'MsgAddMargin',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGADDMARGIN,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgAddMargin)
    },
)
_sym_db.RegisterMessage(MsgAddMargin)

MsgAddMarginResponse = _reflection.GeneratedProtocolMessageType(
    'MsgAddMarginResponse',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGADDMARGINRESPONSE,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgAddMarginResponse)
    },
)
_sym_db.RegisterMessage(MsgAddMarginResponse)

MsgLiquidate = _reflection.GeneratedProtocolMessageType(
    'MsgLiquidate',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGLIQUIDATE,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgLiquidate)
    },
)
_sym_db.RegisterMessage(MsgLiquidate)

MsgLiquidateResponse = _reflection.GeneratedProtocolMessageType(
    'MsgLiquidateResponse',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGLIQUIDATERESPONSE,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgLiquidateResponse)
    },
)
_sym_db.RegisterMessage(MsgLiquidateResponse)

MsgOpenPosition = _reflection.GeneratedProtocolMessageType(
    'MsgOpenPosition',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGOPENPOSITION,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgOpenPosition)
    },
)
_sym_db.RegisterMessage(MsgOpenPosition)

MsgOpenPositionResponse = _reflection.GeneratedProtocolMessageType(
    'MsgOpenPositionResponse',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGOPENPOSITIONRESPONSE,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgOpenPositionResponse)
    },
)
_sym_db.RegisterMessage(MsgOpenPositionResponse)

MsgClosePosition = _reflection.GeneratedProtocolMessageType(
    'MsgClosePosition',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGCLOSEPOSITION,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgClosePosition)
    },
)
_sym_db.RegisterMessage(MsgClosePosition)

MsgClosePositionResponse = _reflection.GeneratedProtocolMessageType(
    'MsgClosePositionResponse',
    (_message.Message,),
    {
        'DESCRIPTOR': _MSGCLOSEPOSITIONRESPONSE,
        '__module__': 'perp.v1.tx_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.perp.v1.MsgClosePositionResponse)
    },
)
_sym_db.RegisterMessage(MsgClosePositionResponse)

_MSG = DESCRIPTOR.services_by_name['Msg']
if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'Z*github.com/NibiruChain/nibiru/x/perp/types'
    _MSGREMOVEMARGIN.fields_by_name['margin']._options = None
    _MSGREMOVEMARGIN.fields_by_name['margin']._serialized_options = b'\310\336\037\000'
    _MSGREMOVEMARGINRESPONSE.fields_by_name['margin_out']._options = None
    _MSGREMOVEMARGINRESPONSE.fields_by_name['margin_out']._serialized_options = b'\310\336\037\000'
    _MSGREMOVEMARGINRESPONSE.fields_by_name['funding_payment']._options = None
    _MSGREMOVEMARGINRESPONSE.fields_by_name[
        'funding_payment'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSGADDMARGIN.fields_by_name['margin']._options = None
    _MSGADDMARGIN.fields_by_name['margin']._serialized_options = b'\310\336\037\000'
    _MSGADDMARGINRESPONSE.fields_by_name['funding_payment']._options = None
    _MSGADDMARGINRESPONSE.fields_by_name[
        'funding_payment'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSGLIQUIDATERESPONSE.fields_by_name['fee_to_liquidator']._options = None
    _MSGLIQUIDATERESPONSE.fields_by_name['fee_to_liquidator']._serialized_options = b'\310\336\037\000'
    _MSGLIQUIDATERESPONSE.fields_by_name['fee_to_perp_ecosystem_fund']._options = None
    _MSGLIQUIDATERESPONSE.fields_by_name['fee_to_perp_ecosystem_fund']._serialized_options = b'\310\336\037\000'
    _MSGOPENPOSITION.fields_by_name['quote_asset_amount']._options = None
    _MSGOPENPOSITION.fields_by_name[
        'quote_asset_amount'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Int\310\336\037\000'
    _MSGOPENPOSITION.fields_by_name['leverage']._options = None
    _MSGOPENPOSITION.fields_by_name[
        'leverage'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSGOPENPOSITION.fields_by_name['base_asset_amount_limit']._options = None
    _MSGOPENPOSITION.fields_by_name[
        'base_asset_amount_limit'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Int\310\336\037\000'
    _MSGOPENPOSITIONRESPONSE.fields_by_name['exchanged_notional_value']._options = None
    _MSGOPENPOSITIONRESPONSE.fields_by_name[
        'exchanged_notional_value'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSGOPENPOSITIONRESPONSE.fields_by_name['exchanged_position_size']._options = None
    _MSGOPENPOSITIONRESPONSE.fields_by_name[
        'exchanged_position_size'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSGOPENPOSITIONRESPONSE.fields_by_name['funding_payment']._options = None
    _MSGOPENPOSITIONRESPONSE.fields_by_name[
        'funding_payment'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSGOPENPOSITIONRESPONSE.fields_by_name['realized_pnl']._options = None
    _MSGOPENPOSITIONRESPONSE.fields_by_name[
        'realized_pnl'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSGOPENPOSITIONRESPONSE.fields_by_name['unrealized_pnl_after']._options = None
    _MSGOPENPOSITIONRESPONSE.fields_by_name[
        'unrealized_pnl_after'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSGOPENPOSITIONRESPONSE.fields_by_name['margin_to_vault']._options = None
    _MSGOPENPOSITIONRESPONSE.fields_by_name[
        'margin_to_vault'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSGOPENPOSITIONRESPONSE.fields_by_name['position_notional']._options = None
    _MSGOPENPOSITIONRESPONSE.fields_by_name[
        'position_notional'
    ]._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
    _MSG.methods_by_name['RemoveMargin']._options = None
    _MSG.methods_by_name[
        'RemoveMargin'
    ]._serialized_options = b'\202\323\344\223\002\034\"\032/nibiru/perp/remove_margin'
    _MSG.methods_by_name['AddMargin']._options = None
    _MSG.methods_by_name['AddMargin']._serialized_options = b'\202\323\344\223\002\031\"\027/nibiru/perp/add_margin'
    _MSG.methods_by_name['Liquidate']._options = None
    _MSG.methods_by_name['Liquidate']._serialized_options = b'\202\323\344\223\002\030\"\026/nibiru/perp/liquidate'
    _MSG.methods_by_name['OpenPosition']._options = None
    _MSG.methods_by_name[
        'OpenPosition'
    ]._serialized_options = b'\202\323\344\223\002\034\"\032/nibiru/perp/open_position'
    _MSG.methods_by_name['ClosePosition']._options = None
    _MSG.methods_by_name[
        'ClosePosition'
    ]._serialized_options = b'\202\323\344\223\002\035\"\033/nibiru/perp/close_position'
    _MSGREMOVEMARGIN._serialized_start = 141
    _MSGREMOVEMARGIN._serialized_end = 243
    _MSGREMOVEMARGINRESPONSE._serialized_start = 246
    _MSGREMOVEMARGINRESPONSE._serialized_end = 397
    _MSGADDMARGIN._serialized_start = 399
    _MSGADDMARGIN._serialized_end = 498
    _MSGADDMARGINRESPONSE._serialized_start = 501
    _MSGADDMARGINRESPONSE._serialized_end = 640
    _MSGLIQUIDATE._serialized_start = 642
    _MSGLIQUIDATE._serialized_end = 708
    _MSGLIQUIDATERESPONSE._serialized_start = 711
    _MSGLIQUIDATERESPONSE._serialized_end = 862
    _MSGOPENPOSITION._serialized_start = 865
    _MSGOPENPOSITION._serialized_end = 1177
    _MSGOPENPOSITIONRESPONSE._serialized_start = 1180
    _MSGOPENPOSITIONRESPONSE._serialized_end = 1781
    _MSGCLOSEPOSITION._serialized_start = 1783
    _MSGCLOSEPOSITION._serialized_end = 1837
    _MSGCLOSEPOSITIONRESPONSE._serialized_start = 1839
    _MSGCLOSEPOSITIONRESPONSE._serialized_end = 1865
    _MSG._serialized_start = 1868
    _MSG._serialized_end = 2483
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dex/v1/params.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x13\x64\x65x/v1/params.proto\x12\rnibiru.dex.v1\x1a\x14gogoproto/gogo.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a\x19\x63osmos_proto/cosmos.proto\"\xcc\x01\n\x06Params\x12\x1c\n\x14starting_pool_number\x18\x01 \x01(\x04\x12\x82\x01\n\x11pool_creation_fee\x18\x02 \x03(\x0b\x32\x19.cosmos.base.v1beta1.CoinBL\xaa\xdf\x1f(github.com/cosmos/cosmos-sdk/types.Coins\xf2\xde\x1f\x18yaml:\"pool_creation_fee\"\xc8\xde\x1f\x00\x12\x19\n\x11whitelisted_asset\x18\x03 \x03(\t:\x04\x98\xa0\x1f\x00\x42+Z)github.com/NibiruChain/nibiru/x/dex/typesb\x06proto3'
)


_PARAMS = DESCRIPTOR.message_types_by_name['Params']
Params = _reflection.GeneratedProtocolMessageType(
    'Params',
    (_message.Message,),
    {
        'DESCRIPTOR': _PARAMS,
        '__module__': 'dex.v1.params_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.dex.v1.Params)
    },
)
_sym_db.RegisterMessage(Params)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'Z)github.com/NibiruChain/nibiru/x/dex/types'
    _PARAMS.fields_by_name['pool_creation_fee']._options = None
    _PARAMS.fields_by_name[
        'pool_creation_fee'
    ]._serialized_options = b'\252\337\037(github.com/cosmos/cosmos-sdk/types.Coins\362\336\037\030yaml:\"pool_creation_fee\"\310\336\037\000'
    _PARAMS._options = None
    _PARAMS._serialized_options = b'\230\240\037\000'
    _PARAMS._serialized_start = 120
    _PARAMS._serialized_end = 324
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stablecoin/params.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x17stablecoin/params.proto\x12\x14nibiru.stablecoin.v1\x1a\x14gogoproto/gogo.proto\"\x95\x02\n\x06Params\x12\x12\n\ncoll_ratio\x18\x01 \x01(\x03\x12\x11\n\tfee_ratio\x18\x02 \x01(\x03\x12\x14\n\x0c\x65\x66_fee_ratio\x18\x03 \x01(\x03\x12\x19\n\x11\x62onus_rate_recoll\x18\x04 \x01(\x03\x12\x41\n\x16\x64istr_epoch_identifier\x18\x05 \x01(\tB!\xf2\xde\x1f\x1dyaml:\"distr_epoch_identifier\"\x12\x17\n\x0f\x61\x64justment_step\x18\x06 \x01(\x03\x12\x19\n\x11price_lower_bound\x18\x07 \x01(\x03\x12\x19\n\x11price_upper_bound\x18\x08 \x01(\x03\x12!\n\x19is_collateral_ratio_valid\x18\t \x01(\x08\x42\x32Z0github.com/NibiruChain/nibiru/x/stablecoin/typesb\x06proto3'
)


_PARAMS = DESCRIPTOR.message_types_by_name['Params']
Params = _reflection.GeneratedProtocolMessageType(
    'Params',
    (_message.Message,),
    {
        'DESCRIPTOR': _PARAMS,
        '__module__': 'stablecoin.params_pb2'
        # @@protoc_insertion_point(class_scope:nibiru.stablecoin.v1.Params)
    },
)
_sym_db.RegisterMessage(Params)

if _descriptor._USE_C_DESCRIPTORS == False:

    DESCRIPTOR._options = None
    DESCRIPTOR._serialized_options = b'Z0github.com/NibiruChain/nibiru/x/stablecoin/types'
    _PARAMS.fields_by_name['distr_epoch_identifier']._options = None
    _PARAMS.fields_by_name[
        'distr_epoch_identifier'
    ]._serialized_options = b'\362\336\037\035yaml:\"distr_epoch_identifier\"'
    _PARAMS._serialized_start = 72
    _PARAMS._serialized_end = 349
# @@protoc_insertion_point(module_scope)



from nibiru.utils import sdkdec_to_float, sdkint_to_float

BASE_ATTRS = [
    'ByteSize',
    'Clear',
    'ClearExtension',
    'ClearField',
    'CopyFrom',
    'DESCRIPTOR',
    'DiscardUnknownFields',
    'Extensions',
    'FindInitializationErrors',
    'FromString',
    'HasExtension',
    'HasField',
    'IsInitialized',
    'ListFields',
    'MergeFrom',
    'MergeFromString',
    'ParseFromString',
    'RegisterExtension',
    'SerializePartialToString',
    'SerializeToString',
    'SetInParent',
    'UnknownFields',
    'WhichOneof',
    '_CheckCalledFromGeneratedFile',
    '_SetListener',
    '__class__',
    '__deepcopy__',
    '__delattr__',
    '__dir__',
    '__doc__',
    '__eq__',
    '__format__',
    '__ge__',
    '__getattribute__',
    '__getstate__',
    '__gt__',
    '__hash__',
    '__init__',
    '__init_subclass__',
    '__le__',
    '__lt__',
    '__module__',
    '__ne__',
    '__new__',
    '__reduce__',
    '__reduce_ex__',
    '__repr__',
    '__setattr__',
    '__setstate__',
    '__sizeof__',
    '__slots__',
    '__str__',
    '__subclasshook__',
    '__unicode__',
    '_extensions_by_name',
    '_extensions_by_number',
]


def deserialize(obj: object) -> dict:
    custom_dtypes = {
        str(field[1]): field[0].GetOptions().__getstate__().get("serialized", None) for field in obj.ListFields()
    }

    serialized_output = {}

    for attr in dir(obj):
        if attr not in BASE_ATTRS:

            attr_search = obj.__getattribute__(attr)

            custom_dtype = custom_dtypes.get(str(attr_search))

            if custom_dtype is not None:

                if "cosmos/cosmos-sdk/types.Dec" in str(custom_dtype):
                    serialized_output[str(attr)] = sdkdec_to_float(obj.__getattribute__(attr))

                elif "cosmos/cosmos-sdk/types.Int" in str(custom_dtype):
                    serialized_output[str(attr)] = sdkint_to_float(obj.__getattribute__(attr))
                else:
                    try:
                        val = obj.__getattribute__(attr)
                        if hasattr(val, '__len__') and not isinstance(val, str):
                            updated_vals = []
                            for v in val:
                                updated_vals.append(deserialize(v))
                            serialized_output[str(attr)] = updated_vals
                        else:
                            serialized_output[str(attr)] = deserialize(val)
                    except:
                        serialized_output[str(attr)] = obj.__getattribute__(attr)
            else:
                serialized_output[str(attr)] = deserialize(obj.__getattribute__(attr))

    return serialized_output



from typing import List

from grpc import Channel

from nibiru.proto.cosmos.base.query.v1beta1.pagination_pb2 import PageRequest
from nibiru.proto.cosmos.base.v1beta1 import coin_pb2 as cosmos_base_coin_pb
from nibiru.proto.dex.v1 import query_pb2 as dex_type
from nibiru.proto.dex.v1 import query_pb2_grpc as dex_query

from .util import deserialize


class Dex:
    """
    Dex allows to query the endpoints made available by the Nibiru Chain's DEX module.
    """

    def __init__(self, channel: Channel):
        self.api = dex_query.QueryStub(channel)

    def params(self):
        req = dex_type.QueryParamsRequest()
        return deserialize(self.api.Params(req))

    def pool_number(self):
        req = dex_type.QueryPoolsRequest()
        return deserialize(self.api.PoolNumber(req))

    def pool(self, pool_id: int):
        req = dex_type.QueryPoolRequest(poolId=pool_id)
        return deserialize(self.api.Pool(req))

    def pools(self, **kwargs):
        '''
        Returns all available pools
        Parameters:
            key (bytes): The page key for the next page. Only key or offset should be set
            offset (int): The number of entries to skip. Only offset or key should be set
            limit (int): The number of max results in the page
            count_total (bool): Indicates if the response should contain the total number of results
            reverse (bool): Indicates if the results should be returned in descending order
        '''
        req = dex_type.QueryPoolsRequest(
            pagination=PageRequest(
                key=kwargs.get("key"),
                offset=kwargs.get("offset"),
                limit=kwargs.get("limit"),
                count_total=kwargs.get("count_total"),
                reverse=kwargs.get("reverse"),
            ),
        )
        return deserialize(self.api.Pools(req))

    def pool_params(self, pool_id: int):
        req = dex_type.QueryPoolParamsRequest(poolId=pool_id)
        return deserialize(self.api.PoolParams(req))

    def num_pools(self):
        req = dex_type.QueryNumPoolsRequest()
        return deserialize(self.api.NumPools(req))

    def total_liquidity(self):
        req = dex_type.QueryTotalLiquidityRequest()
        return deserialize(self.api.TotalLiquidity(req))

    def total_pool_liquidity(self, pool_id: int):
        req = dex_type.QueryTotalPoolLiquidityRequest(poolId=pool_id)
        return deserialize(self.api.TotalPoolLiquidity(req))

    def total_shares(self, pool_id: int):
        req = dex_type.QueryTotalSharesRequest(poolId=pool_id)
        return deserialize(self.api.TotalShares(req))

    def spot_price(self, pool_id: int, token_in_denom: str, token_out_denom: str):
        req = dex_type.QuerySpotPriceRequest(
            pool_id=pool_id,
            token_in_denom=token_in_denom,
            token_out_denom=token_out_denom,
        )
        return deserialize(self.api.SpotPrice(req))

    def estimate_swap_exact_amount_in(self, pool_id: int, token_in: cosmos_base_coin_pb.Coin, token_out_denom: str):
        req = dex_type.QuerySwapExactAmountInRequest(
            pool_id=pool_id,
            token_in=token_in,
            token_out_denom=token_out_denom,
        )
        return deserialize(self.api.EstimateSwapExactAmountIn(req))

    def estimate_swap_exact_amount_out(self, pool_id: int, token_out: cosmos_base_coin_pb.Coin, token_in_denom: str):
        req = dex_type.QuerySwapExactAmountOutRequest(
            pool_id=pool_id,
            token_out=token_out,
            token_in_denom=token_in_denom,
        )
        return deserialize(self.api.EstimateSwapExactAmountOut(req))

    def estimate_join_exact_amount_in(self, pool_id: int, tokens_in: List[cosmos_base_coin_pb.Coin]):
        req = dex_type.QueryJoinExactAmountInRequest(
            pool_id=pool_id,
            tokens_in=tokens_in,
        )
        return deserialize(self.api.EstimateJoinExactAmountIn(req))

    def estimate_join_exact_amount_out(self, pool_id: int):
        req = dex_type.QueryJoinExactAmountOutRequest(
            pool_id=pool_id,
        )
        return deserialize(self.api.EstimateJoinExactAmountOut(req))

    def estimate_exit_exact_amount_in(self, pool_id: int, num_shares: int):
        req = dex_type.QueryExitExactAmountInRequest(pool_id=pool_id, pool_shares_in=num_shares)
        return deserialize(self.api.EstimateExitExactAmountIn(req))

    def estimate_exit_exact_amount_out(self, pool_id: int):
        req = dex_type.QueryExitExactAmountOutRequest(
            pool_id=pool_id,
        )
        return deserialize(self.api.EstimateExitExactAmountOut(req))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gogoproto/gogo.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14gogoproto/gogo.proto\x12\tgogoproto\x1a google/protobuf/descriptor.proto:;\n\x13goproto_enum_prefix\x12\x1c.google.protobuf.EnumOptions\x18\xb1\xe4\x03 \x01(\x08:=\n\x15goproto_enum_stringer\x12\x1c.google.protobuf.EnumOptions\x18\xc5\xe4\x03 \x01(\x08:5\n\renum_stringer\x12\x1c.google.protobuf.EnumOptions\x18\xc6\xe4\x03 \x01(\x08:7\n\x0f\x65num_customname\x12\x1c.google.protobuf.EnumOptions\x18\xc7\xe4\x03 \x01(\t:0\n\x08\x65numdecl\x12\x1c.google.protobuf.EnumOptions\x18\xc8\xe4\x03 \x01(\x08:A\n\x14\x65numvalue_customname\x12!.google.protobuf.EnumValueOptions\x18\xd1\x83\x04 \x01(\t:;\n\x13goproto_getters_all\x12\x1c.google.protobuf.FileOptions\x18\x99\xec\x03 \x01(\x08:?\n\x17goproto_enum_prefix_all\x12\x1c.google.protobuf.FileOptions\x18\x9a\xec\x03 \x01(\x08:<\n\x14goproto_stringer_all\x12\x1c.google.protobuf.FileOptions\x18\x9b\xec\x03 \x01(\x08:9\n\x11verbose_equal_all\x12\x1c.google.protobuf.FileOptions\x18\x9c\xec\x03 \x01(\x08:0\n\x08\x66\x61\x63\x65_all\x12\x1c.google.protobuf.FileOptions\x18\x9d\xec\x03 \x01(\x08:4\n\x0cgostring_all\x12\x1c.google.protobuf.FileOptions\x18\x9e\xec\x03 \x01(\x08:4\n\x0cpopulate_all\x12\x1c.google.protobuf.FileOptions\x18\x9f\xec\x03 \x01(\x08:4\n\x0cstringer_all\x12\x1c.google.protobuf.FileOptions\x18\xa0\xec\x03 \x01(\x08:3\n\x0bonlyone_all\x12\x1c.google.protobuf.FileOptions\x18\xa1\xec\x03 \x01(\x08:1\n\tequal_all\x12\x1c.google.protobuf.FileOptions\x18\xa5\xec\x03 \x01(\x08:7\n\x0f\x64\x65scription_all\x12\x1c.google.protobuf.FileOptions\x18\xa6\xec\x03 \x01(\x08:3\n\x0btestgen_all\x12\x1c.google.protobuf.FileOptions\x18\xa7\xec\x03 \x01(\x08:4\n\x0c\x62\x65nchgen_all\x12\x1c.google.protobuf.FileOptions\x18\xa8\xec\x03 \x01(\x08:5\n\rmarshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xa9\xec\x03 \x01(\x08:7\n\x0funmarshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xaa\xec\x03 \x01(\x08:<\n\x14stable_marshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xab\xec\x03 \x01(\x08:1\n\tsizer_all\x12\x1c.google.protobuf.FileOptions\x18\xac\xec\x03 \x01(\x08:A\n\x19goproto_enum_stringer_all\x12\x1c.google.protobuf.FileOptions\x18\xad\xec\x03 \x01(\x08:9\n\x11\x65num_stringer_all\x12\x1c.google.protobuf.FileOptions\x18\xae\xec\x03 \x01(\x08:<\n\x14unsafe_marshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xaf\xec\x03 \x01(\x08:>\n\x16unsafe_unmarshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xb0\xec\x03 \x01(\x08:B\n\x1agoproto_extensions_map_all\x12\x1c.google.protobuf.FileOptions\x18\xb1\xec\x03 \x01(\x08:@\n\x18goproto_unrecognized_all\x12\x1c.google.protobuf.FileOptions\x18\xb2\xec\x03 \x01(\x08:8\n\x10gogoproto_import\x12\x1c.google.protobuf.FileOptions\x18\xb3\xec\x03 \x01(\x08:6\n\x0eprotosizer_all\x12\x1c.google.protobuf.FileOptions\x18\xb4\xec\x03 \x01(\x08:3\n\x0b\x63ompare_all\x12\x1c.google.protobuf.FileOptions\x18\xb5\xec\x03 \x01(\x08:4\n\x0ctypedecl_all\x12\x1c.google.protobuf.FileOptions\x18\xb6\xec\x03 \x01(\x08:4\n\x0c\x65numdecl_all\x12\x1c.google.protobuf.FileOptions\x18\xb7\xec\x03 \x01(\x08:<\n\x14goproto_registration\x12\x1c.google.protobuf.FileOptions\x18\xb8\xec\x03 \x01(\x08:7\n\x0fmessagename_all\x12\x1c.google.protobuf.FileOptions\x18\xb9\xec\x03 \x01(\x08:=\n\x15goproto_sizecache_all\x12\x1c.google.protobuf.FileOptions\x18\xba\xec\x03 \x01(\x08:;\n\x13goproto_unkeyed_all\x12\x1c.google.protobuf.FileOptions\x18\xbb\xec\x03 \x01(\x08::\n\x0fgoproto_getters\x12\x1f.google.protobuf.MessageOptions\x18\x81\xf4\x03 \x01(\x08:;\n\x10goproto_stringer\x12\x1f.google.protobuf.MessageOptions\x18\x83\xf4\x03 \x01(\x08:8\n\rverbose_equal\x12\x1f.google.protobuf.MessageOptions\x18\x84\xf4\x03 \x01(\x08:/\n\x04\x66\x61\x63\x65\x12\x1f.google.protobuf.MessageOptions\x18\x85\xf4\x03 \x01(\x08:3\n\x08gostring\x12\x1f.google.protobuf.MessageOptions\x18\x86\xf4\x03 \x01(\x08:3\n\x08populate\x12\x1f.google.protobuf.MessageOptions\x18\x87\xf4\x03 \x01(\x08:3\n\x08stringer\x12\x1f.google.protobuf.MessageOptions\x18\xc0\x8b\x04 \x01(\x08:2\n\x07onlyone\x12\x1f.google.protobuf.MessageOptions\x18\x89\xf4\x03 \x01(\x08:0\n\x05\x65qual\x12\x1f.google.protobuf.MessageOptions\x18\x8d\xf4\x03 \x01(\x08:6\n\x0b\x64\x65scription\x12\x1f.google.protobuf.MessageOptions\x18\x8e\xf4\x03 \x01(\x08:2\n\x07testgen\x12\x1f.google.protobuf.MessageOptions\x18\x8f\xf4\x03 \x01(\x08:3\n\x08\x62\x65nchgen\x12\x1f.google.protobuf.MessageOptions\x18\x90\xf4\x03 \x01(\x08:4\n\tmarshaler\x12\x1f.google.protobuf.MessageOptions\x18\x91\xf4\x03 \x01(\x08:6\n\x0bunmarshaler\x12\x1f.google.protobuf.MessageOptions\x18\x92\xf4\x03 \x01(\x08:;\n\x10stable_marshaler\x12\x1f.google.protobuf.MessageOptions\x18\x93\xf4\x03 \x01(\x08:0\n\x05sizer\x12\x1f.google.protobuf.MessageOptions\x18\x94\xf4\x03 \x01(\x08:;\n\x10unsafe_marshaler\x12\x1f.google.protobuf.MessageOptions\x18\x97\xf4\x03 \x01(\x08:=\n\x12unsafe_unmarshaler\x12\x1f.google.protobuf.MessageOptions\x18\x98\xf4\x03 \x01(\x08:A\n\x16goproto_extensions_map\x12\x1f.google.protobuf.MessageOptions\x18\x99\xf4\x03 \x01(\x08:?\n\x14goproto_unrecognized\x12\x1f.google.protobuf.MessageOptions\x18\x9a\xf4\x03 \x01(\x08:5\n\nprotosizer\x12\x1f.google.protobuf.MessageOptions\x18\x9c\xf4\x03 \x01(\x08:2\n\x07\x63ompare\x12\x1f.google.protobuf.MessageOptions\x18\x9d\xf4\x03 \x01(\x08:3\n\x08typedecl\x12\x1f.google.protobuf.MessageOptions\x18\x9e\xf4\x03 \x01(\x08:6\n\x0bmessagename\x12\x1f.google.protobuf.MessageOptions\x18\xa1\xf4\x03 \x01(\x08:<\n\x11goproto_sizecache\x12\x1f.google.protobuf.MessageOptions\x18\xa2\xf4\x03 \x01(\x08::\n\x0fgoproto_unkeyed\x12\x1f.google.protobuf.MessageOptions\x18\xa3\xf4\x03 \x01(\x08:1\n\x08nullable\x12\x1d.google.protobuf.FieldOptions\x18\xe9\xfb\x03 \x01(\x08:.\n\x05\x65mbed\x12\x1d.google.protobuf.FieldOptions\x18\xea\xfb\x03 \x01(\x08:3\n\ncustomtype\x12\x1d.google.protobuf.FieldOptions\x18\xeb\xfb\x03 \x01(\t:3\n\ncustomname\x12\x1d.google.protobuf.FieldOptions\x18\xec\xfb\x03 \x01(\t:0\n\x07jsontag\x12\x1d.google.protobuf.FieldOptions\x18\xed\xfb\x03 \x01(\t:1\n\x08moretags\x12\x1d.google.protobuf.FieldOptions\x18\xee\xfb\x03 \x01(\t:1\n\x08\x63\x61sttype\x12\x1d.google.protobuf.FieldOptions\x18\xef\xfb\x03 \x01(\t:0\n\x07\x63\x61stkey\x12\x1d.google.protobuf.FieldOptions\x18\xf0\xfb\x03 \x01(\t:2\n\tcastvalue\x12\x1d.google.protobuf.FieldOptions\x18\xf1\xfb\x03 \x01(\t:0\n\x07stdtime\x12\x1d.google.protobuf.FieldOptions\x18\xf2\xfb\x03 \x01(\x08:4\n\x0bstdduration\x12\x1d.google.protobuf.FieldOptions\x18\xf3\xfb\x03 \x01(\x08:3\n\nwktpointer\x12\x1d.google.protobuf.FieldOptions\x18\xf4\xfb\x03 \x01(\x08:5\n\x0c\x63\x61strepeated\x12\x1d.google.protobuf.FieldOptions\x18\xf5\xfb\x03 \x01(\tBE\n\x13\x63om.google.protobufB\nGoGoProtosZ\"github.com/gogo/protobuf/gogoproto')


GOPROTO_ENUM_PREFIX_FIELD_NUMBER = 62001
goproto_enum_prefix = DESCRIPTOR.extensions_by_name['goproto_enum_prefix']
GOPROTO_ENUM_STRINGER_FIELD_NUMBER = 62021
goproto_enum_stringer = DESCRIPTOR.extensions_by_name['goproto_enum_stringer']
ENUM_STRINGER_FIELD_NUMBER = 62022
enum_stringer = DESCRIPTOR.extensions_by_name['enum_stringer']
ENUM_CUSTOMNAME_FIELD_NUMBER = 62023
enum_customname = DESCRIPTOR.extensions_by_name['enum_customname']
ENUMDECL_FIELD_NUMBER = 62024
enumdecl = DESCRIPTOR.extensions_by_name['enumdecl']
ENUMVALUE_CUSTOMNAME_FIELD_NUMBER = 66001
enumvalue_customname = DESCRIPTOR.extensions_by_name['enumvalue_customname']
GOPROTO_GETTERS_ALL_FIELD_NUMBER = 63001
goproto_getters_all = DESCRIPTOR.extensions_by_name['goproto_getters_all']
GOPROTO_ENUM_PREFIX_ALL_FIELD_NUMBER = 63002
goproto_enum_prefix_all = DESCRIPTOR.extensions_by_name['goproto_enum_prefix_all']
GOPROTO_STRINGER_ALL_FIELD_NUMBER = 63003
goproto_stringer_all = DESCRIPTOR.extensions_by_name['goproto_stringer_all']
VERBOSE_EQUAL_ALL_FIELD_NUMBER = 63004
verbose_equal_all = DESCRIPTOR.extensions_by_name['verbose_equal_all']
FACE_ALL_FIELD_NUMBER = 63005
face_all = DESCRIPTOR.extensions_by_name['face_all']
GOSTRING_ALL_FIELD_NUMBER = 63006
gostring_all = DESCRIPTOR.extensions_by_name['gostring_all']
POPULATE_ALL_FIELD_NUMBER = 63007
populate_all = DESCRIPTOR.extensions_by_name['populate_all']
STRINGER_ALL_FIELD_NUMBER = 63008
stringer_all = DESCRIPTOR.extensions_by_name['stringer_all']
ONLYONE_ALL_FIELD_NUMBER = 63009
onlyone_all = DESCRIPTOR.extensions_by_name['onlyone_all']
EQUAL_ALL_FIELD_NUMBER = 63013
equal_all = DESCRIPTOR.extensions_by_name['equal_all']
DESCRIPTION_ALL_FIELD_NUMBER = 63014
description_all = DESCRIPTOR.extensions_by_name['description_all']
TESTGEN_ALL_FIELD_NUMBER = 63015
testgen_all = DESCRIPTOR.extensions_by_name['testgen_all']
BENCHGEN_ALL_FIELD_NUMBER = 63016
benchgen_all = DESCRIPTOR.extensions_by_name['benchgen_all']
MARSHALER_ALL_FIELD_NUMBER = 63017
marshaler_all = DESCRIPTOR.extensions_by_name['marshaler_all']
UNMARSHALER_ALL_FIELD_NUMBER = 63018
unmarshaler_all = DESCRIPTOR.extensions_by_name['unmarshaler_all']
STABLE_MARSHALER_ALL_FIELD_NUMBER = 63019
stable_marshaler_all = DESCRIPTOR.extensions_by_name['stable_marshaler_all']
SIZER_ALL_FIELD_NUMBER = 63020
sizer_all = DESCRIPTOR.extensions_by_name['sizer_all']
GOPROTO_ENUM_STRINGER_ALL_FIELD_NUMBER = 63021
goproto_enum_stringer_all = DESCRIPTOR.extensions_by_name['goproto_enum_stringer_all']
ENUM_STRINGER_ALL_FIELD_NUMBER = 63022
enum_stringer_all = DESCRIPTOR.extensions_by_name['enum_stringer_all']
UNSAFE_MARSHALER_ALL_FIELD_NUMBER = 63023
unsafe_marshaler_all = DESCRIPTOR.extensions_by_name['unsafe_marshaler_all']
UNSAFE_UNMARSHALER_ALL_FIELD_NUMBER = 63024
unsafe_unmarshaler_all = DESCRIPTOR.extensions_by_name['unsafe_unmarshaler_all']
GOPROTO_EXTENSIONS_MAP_ALL_FIELD_NUMBER = 63025
goproto_extensions_map_all = DESCRIPTOR.extensions_by_name['goproto_extensions_map_all']
GOPROTO_UNRECOGNIZED_ALL_FIELD_NUMBER = 63026
goproto_unrecognized_all = DESCRIPTOR.extensions_by_name['goproto_unrecognized_all']
GOGOPROTO_IMPORT_FIELD_NUMBER = 63027
gogoproto_import = DESCRIPTOR.extensions_by_name['gogoproto_import']
PROTOSIZER_ALL_FIELD_NUMBER = 63028
protosizer_all = DESCRIPTOR.extensions_by_name['protosizer_all']
COMPARE_ALL_FIELD_NUMBER = 63029
compare_all = DESCRIPTOR.extensions_by_name['compare_all']
TYPEDECL_ALL_FIELD_NUMBER = 63030
typedecl_all = DESCRIPTOR.extensions_by_name['typedecl_all']
ENUMDECL_ALL_FIELD_NUMBER = 63031
enumdecl_all = DESCRIPTOR.extensions_by_name['enumdecl_all']
GOPROTO_REGISTRATION_FIELD_NUMBER = 63032
goproto_registration = DESCRIPTOR.extensions_by_name['goproto_registration']
MESSAGENAME_ALL_FIELD_NUMBER = 63033
messagename_all = DESCRIPTOR.extensions_by_name['messagename_all']
GOPROTO_SIZECACHE_ALL_FIELD_NUMBER = 63034
goproto_sizecache_all = DESCRIPTOR.extensions_by_name['goproto_sizecache_all']
GOPROTO_UNKEYED_ALL_FIELD_NUMBER = 63035
goproto_unkeyed_all = DESCRIPTOR.extensions_by_name['goproto_unkeyed_all']
GOPROTO_GETTERS_FIELD_NUMBER = 64001
goproto_getters = DESCRIPTOR.extensions_by_name['goproto_getters']
GOPROTO_STRINGER_FIELD_NUMBER = 64003
goproto_stringer = DESCRIPTOR.extensions_by_name['goproto_stringer']
VERBOSE_EQUAL_FIELD_NUMBER = 64004
verbose_equal = DESCRIPTOR.extensions_by_name['verbose_equal']
FACE_FIELD_NUMBER = 64005
face = DESCRIPTOR.extensions_by_name['face']
GOSTRING_FIELD_NUMBER = 64006
gostring = DESCRIPTOR.extensions_by_name['gostring']
POPULATE_FIELD_NUMBER = 64007
populate = DESCRIPTOR.extensions_by_name['populate']
STRINGER_FIELD_NUMBER = 67008
stringer = DESCRIPTOR.extensions_by_name['stringer']
ONLYONE_FIELD_NUMBER = 64009
onlyone = DESCRIPTOR.extensions_by_name['onlyone']
EQUAL_FIELD_NUMBER = 64013
equal = DESCRIPTOR.extensions_by_name['equal']
DESCRIPTION_FIELD_NUMBER = 64014
description = DESCRIPTOR.extensions_by_name['description']
TESTGEN_FIELD_NUMBER = 64015
testgen = DESCRIPTOR.extensions_by_name['testgen']
BENCHGEN_FIELD_NUMBER = 64016
benchgen = DESCRIPTOR.extensions_by_name['benchgen']
MARSHALER_FIELD_NUMBER = 64017
marshaler = DESCRIPTOR.extensions_by_name['marshaler']
UNMARSHALER_FIELD_NUMBER = 64018
unmarshaler = DESCRIPTOR.extensions_by_name['unmarshaler']
STABLE_MARSHALER_FIELD_NUMBER = 64019
stable_marshaler = DESCRIPTOR.extensions_by_name['stable_marshaler']
SIZER_FIELD_NUMBER = 64020
sizer = DESCRIPTOR.extensions_by_name['sizer']
UNSAFE_MARSHALER_FIELD_NUMBER = 64023
unsafe_marshaler = DESCRIPTOR.extensions_by_name['unsafe_marshaler']
UNSAFE_UNMARSHALER_FIELD_NUMBER = 64024
unsafe_unmarshaler = DESCRIPTOR.extensions_by_name['unsafe_unmarshaler']
GOPROTO_EXTENSIONS_MAP_FIELD_NUMBER = 64025
goproto_extensions_map = DESCRIPTOR.extensions_by_name['goproto_extensions_map']
GOPROTO_UNRECOGNIZED_FIELD_NUMBER = 64026
goproto_unrecognized = DESCRIPTOR.extensions_by_name['goproto_unrecognized']
PROTOSIZER_FIELD_NUMBER = 64028
protosizer = DESCRIPTOR.extensions_by_name['protosizer']
COMPARE_FIELD_NUMBER = 64029
compare = DESCRIPTOR.extensions_by_name['compare']
TYPEDECL_FIELD_NUMBER = 64030
typedecl = DESCRIPTOR.extensions_by_name['typedecl']
MESSAGENAME_FIELD_NUMBER = 64033
messagename = DESCRIPTOR.extensions_by_name['messagename']
GOPROTO_SIZECACHE_FIELD_NUMBER = 64034
goproto_sizecache = DESCRIPTOR.extensions_by_name['goproto_sizecache']
GOPROTO_UNKEYED_FIELD_NUMBER = 64035
goproto_unkeyed = DESCRIPTOR.extensions_by_name['goproto_unkeyed']
NULLABLE_FIELD_NUMBER = 65001
nullable = DESCRIPTOR.extensions_by_name['nullable']
EMBED_FIELD_NUMBER = 65002
embed = DESCRIPTOR.extensions_by_name['embed']
CUSTOMTYPE_FIELD_NUMBER = 65003
customtype = DESCRIPTOR.extensions_by_name['customtype']
CUSTOMNAME_FIELD_NUMBER = 65004
customname = DESCRIPTOR.extensions_by_name['customname']
JSONTAG_FIELD_NUMBER = 65005
jsontag = DESCRIPTOR.extensions_by_name['jsontag']
MORETAGS_FIELD_NUMBER = 65006
moretags = DESCRIPTOR.extensions_by_name['moretags']
CASTTYPE_FIELD_NUMBER = 65007
casttype = DESCRIPTOR.extensions_by_name['casttype']
CASTKEY_FIELD_NUMBER = 65008
castkey = DESCRIPTOR.extensions_by_name['castkey']
CASTVALUE_FIELD_NUMBER = 65009
castvalue = DESCRIPTOR.extensions_by_name['castvalue']
STDTIME_FIELD_NUMBER = 65010
stdtime = DESCRIPTOR.extensions_by_name['stdtime']
STDDURATION_FIELD_NUMBER = 65011
stdduration = DESCRIPTOR.extensions_by_name['stdduration']
WKTPOINTER_FIELD_NUMBER = 65012
wktpointer = DESCRIPTOR.extensions_by_name['wktpointer']
CASTREPEATED_FIELD_NUMBER = 65013
castrepeated = DESCRIPTOR.extensions_by_name['castrepeated']

if _descriptor._USE_C_DESCRIPTORS == False:
  google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(goproto_enum_prefix)
  google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(goproto_enum_stringer)
  google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(enum_stringer)
  google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(enum_customname)
  google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(enumdecl)
  google_dot_protobuf_dot_descriptor__pb2.EnumValueOptions.RegisterExtension(enumvalue_customname)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(goproto_getters_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(goproto_enum_prefix_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(goproto_stringer_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(verbose_equal_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(face_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(gostring_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(populate_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(stringer_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(onlyone_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(equal_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(description_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(testgen_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(benchgen_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(marshaler_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(unmarshaler_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(stable_marshaler_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(sizer_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(goproto_enum_stringer_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(enum_stringer_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(unsafe_marshaler_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(unsafe_unmarshaler_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(goproto_extensions_map_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(goproto_unrecognized_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(gogoproto_import)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(protosizer_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(compare_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(typedecl_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(enumdecl_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(goproto_registration)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(messagename_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(goproto_sizecache_all)
  google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(goproto_unkeyed_all)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(goproto_getters)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(goproto_stringer)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(verbose_equal)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(face)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(gostring)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(populate)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(stringer)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(onlyone)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(equal)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(description)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(testgen)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(benchgen)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(marshaler)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(unmarshaler)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(stable_marshaler)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(sizer)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(unsafe_marshaler)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(unsafe_unmarshaler)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(goproto_extensions_map)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(goproto_unrecognized)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(protosizer)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(compare)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(typedecl)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(messagename)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(goproto_sizecache)
  google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(goproto_unkeyed)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(nullable)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(embed)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(customtype)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(customname)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(jsontag)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(moretags)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(casttype)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(castkey)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(castvalue)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(stdtime)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(stdduration)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(wktpointer)
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(castrepeated)

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.google.protobufB\nGoGoProtosZ\"github.com/gogo/protobuf/gogoproto'
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/base/snapshots/v1beta1/snapshot.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,cosmos/base/snapshots/v1beta1/snapshot.proto\x12\x1d\x63osmos.base.snapshots.v1beta1\x1a\x14gogoproto/gogo.proto\"\x89\x01\n\x08Snapshot\x12\x0e\n\x06height\x18\x01 \x01(\x04\x12\x0e\n\x06\x66ormat\x18\x02 \x01(\r\x12\x0e\n\x06\x63hunks\x18\x03 \x01(\r\x12\x0c\n\x04hash\x18\x04 \x01(\x0c\x12?\n\x08metadata\x18\x05 \x01(\x0b\x32\'.cosmos.base.snapshots.v1beta1.MetadataB\x04\xc8\xde\x1f\x00\" \n\x08Metadata\x12\x14\n\x0c\x63hunk_hashes\x18\x01 \x03(\x0c\"\xc5\x02\n\x0cSnapshotItem\x12\x41\n\x05store\x18\x01 \x01(\x0b\x32\x30.cosmos.base.snapshots.v1beta1.SnapshotStoreItemH\x00\x12I\n\x04iavl\x18\x02 \x01(\x0b\x32/.cosmos.base.snapshots.v1beta1.SnapshotIAVLItemB\x08\xe2\xde\x1f\x04IAVLH\x00\x12I\n\textension\x18\x03 \x01(\x0b\x32\x34.cosmos.base.snapshots.v1beta1.SnapshotExtensionMetaH\x00\x12T\n\x11\x65xtension_payload\x18\x04 \x01(\x0b\x32\x37.cosmos.base.snapshots.v1beta1.SnapshotExtensionPayloadH\x00\x42\x06\n\x04item\"!\n\x11SnapshotStoreItem\x12\x0c\n\x04name\x18\x01 \x01(\t\"O\n\x10SnapshotIAVLItem\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x0f\n\x07version\x18\x03 \x01(\x03\x12\x0e\n\x06height\x18\x04 \x01(\x05\"5\n\x15SnapshotExtensionMeta\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06\x66ormat\x18\x02 \x01(\r\"+\n\x18SnapshotExtensionPayload\x12\x0f\n\x07payload\x18\x01 \x01(\x0c\x42.Z,github.com/cosmos/cosmos-sdk/snapshots/typesb\x06proto3')



_SNAPSHOT = DESCRIPTOR.message_types_by_name['Snapshot']
_METADATA = DESCRIPTOR.message_types_by_name['Metadata']
_SNAPSHOTITEM = DESCRIPTOR.message_types_by_name['SnapshotItem']
_SNAPSHOTSTOREITEM = DESCRIPTOR.message_types_by_name['SnapshotStoreItem']
_SNAPSHOTIAVLITEM = DESCRIPTOR.message_types_by_name['SnapshotIAVLItem']
_SNAPSHOTEXTENSIONMETA = DESCRIPTOR.message_types_by_name['SnapshotExtensionMeta']
_SNAPSHOTEXTENSIONPAYLOAD = DESCRIPTOR.message_types_by_name['SnapshotExtensionPayload']
Snapshot = _reflection.GeneratedProtocolMessageType('Snapshot', (_message.Message,), {
  'DESCRIPTOR' : _SNAPSHOT,
  '__module__' : 'cosmos.base.snapshots.v1beta1.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.snapshots.v1beta1.Snapshot)
  })
_sym_db.RegisterMessage(Snapshot)

Metadata = _reflection.GeneratedProtocolMessageType('Metadata', (_message.Message,), {
  'DESCRIPTOR' : _METADATA,
  '__module__' : 'cosmos.base.snapshots.v1beta1.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.snapshots.v1beta1.Metadata)
  })
_sym_db.RegisterMessage(Metadata)

SnapshotItem = _reflection.GeneratedProtocolMessageType('SnapshotItem', (_message.Message,), {
  'DESCRIPTOR' : _SNAPSHOTITEM,
  '__module__' : 'cosmos.base.snapshots.v1beta1.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.snapshots.v1beta1.SnapshotItem)
  })
_sym_db.RegisterMessage(SnapshotItem)

SnapshotStoreItem = _reflection.GeneratedProtocolMessageType('SnapshotStoreItem', (_message.Message,), {
  'DESCRIPTOR' : _SNAPSHOTSTOREITEM,
  '__module__' : 'cosmos.base.snapshots.v1beta1.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.snapshots.v1beta1.SnapshotStoreItem)
  })
_sym_db.RegisterMessage(SnapshotStoreItem)

SnapshotIAVLItem = _reflection.GeneratedProtocolMessageType('SnapshotIAVLItem', (_message.Message,), {
  'DESCRIPTOR' : _SNAPSHOTIAVLITEM,
  '__module__' : 'cosmos.base.snapshots.v1beta1.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.snapshots.v1beta1.SnapshotIAVLItem)
  })
_sym_db.RegisterMessage(SnapshotIAVLItem)

SnapshotExtensionMeta = _reflection.GeneratedProtocolMessageType('SnapshotExtensionMeta', (_message.Message,), {
  'DESCRIPTOR' : _SNAPSHOTEXTENSIONMETA,
  '__module__' : 'cosmos.base.snapshots.v1beta1.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.snapshots.v1beta1.SnapshotExtensionMeta)
  })
_sym_db.RegisterMessage(SnapshotExtensionMeta)

SnapshotExtensionPayload = _reflection.GeneratedProtocolMessageType('SnapshotExtensionPayload', (_message.Message,), {
  'DESCRIPTOR' : _SNAPSHOTEXTENSIONPAYLOAD,
  '__module__' : 'cosmos.base.snapshots.v1beta1.snapshot_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.snapshots.v1beta1.SnapshotExtensionPayload)
  })
_sym_db.RegisterMessage(SnapshotExtensionPayload)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/cosmos/cosmos-sdk/snapshots/types'
  _SNAPSHOT.fields_by_name['metadata']._options = None
  _SNAPSHOT.fields_by_name['metadata']._serialized_options = b'\310\336\037\000'
  _SNAPSHOTITEM.fields_by_name['iavl']._options = None
  _SNAPSHOTITEM.fields_by_name['iavl']._serialized_options = b'\342\336\037\004IAVL'
  _SNAPSHOT._serialized_start=102
  _SNAPSHOT._serialized_end=239
  _METADATA._serialized_start=241
  _METADATA._serialized_end=273
  _SNAPSHOTITEM._serialized_start=276
  _SNAPSHOTITEM._serialized_end=601
  _SNAPSHOTSTOREITEM._serialized_start=603
  _SNAPSHOTSTOREITEM._serialized_end=636
  _SNAPSHOTIAVLITEM._serialized_start=638
  _SNAPSHOTIAVLITEM._serialized_end=717
  _SNAPSHOTEXTENSIONMETA._serialized_start=719
  _SNAPSHOTEXTENSIONMETA._serialized_end=772
  _SNAPSHOTEXTENSIONPAYLOAD._serialized_start=774
  _SNAPSHOTEXTENSIONPAYLOAD._serialized_end=817
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tendermint/crypto/proof.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dtendermint/crypto/proof.proto\x12\x11tendermint.crypto\x1a\x14gogoproto/gogo.proto\"G\n\x05Proof\x12\r\n\x05total\x18\x01 \x01(\x03\x12\r\n\x05index\x18\x02 \x01(\x03\x12\x11\n\tleaf_hash\x18\x03 \x01(\x0c\x12\r\n\x05\x61unts\x18\x04 \x03(\x0c\"?\n\x07ValueOp\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\'\n\x05proof\x18\x02 \x01(\x0b\x32\x18.tendermint.crypto.Proof\"6\n\x08\x44ominoOp\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05input\x18\x02 \x01(\t\x12\x0e\n\x06output\x18\x03 \x01(\t\"2\n\x07ProofOp\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\"9\n\x08ProofOps\x12-\n\x03ops\x18\x01 \x03(\x0b\x32\x1a.tendermint.crypto.ProofOpB\x04\xc8\xde\x1f\x00\x42:Z8github.com/tendermint/tendermint/proto/tendermint/cryptob\x06proto3')



_PROOF = DESCRIPTOR.message_types_by_name['Proof']
_VALUEOP = DESCRIPTOR.message_types_by_name['ValueOp']
_DOMINOOP = DESCRIPTOR.message_types_by_name['DominoOp']
_PROOFOP = DESCRIPTOR.message_types_by_name['ProofOp']
_PROOFOPS = DESCRIPTOR.message_types_by_name['ProofOps']
Proof = _reflection.GeneratedProtocolMessageType('Proof', (_message.Message,), {
  'DESCRIPTOR' : _PROOF,
  '__module__' : 'tendermint.crypto.proof_pb2'
  # @@protoc_insertion_point(class_scope:tendermint.crypto.Proof)
  })
_sym_db.RegisterMessage(Proof)

ValueOp = _reflection.GeneratedProtocolMessageType('ValueOp', (_message.Message,), {
  'DESCRIPTOR' : _VALUEOP,
  '__module__' : 'tendermint.crypto.proof_pb2'
  # @@protoc_insertion_point(class_scope:tendermint.crypto.ValueOp)
  })
_sym_db.RegisterMessage(ValueOp)

DominoOp = _reflection.GeneratedProtocolMessageType('DominoOp', (_message.Message,), {
  'DESCRIPTOR' : _DOMINOOP,
  '__module__' : 'tendermint.crypto.proof_pb2'
  # @@protoc_insertion_point(class_scope:tendermint.crypto.DominoOp)
  })
_sym_db.RegisterMessage(DominoOp)

ProofOp = _reflection.GeneratedProtocolMessageType('ProofOp', (_message.Message,), {
  'DESCRIPTOR' : _PROOFOP,
  '__module__' : 'tendermint.crypto.proof_pb2'
  # @@protoc_insertion_point(class_scope:tendermint.crypto.ProofOp)
  })
_sym_db.RegisterMessage(ProofOp)

ProofOps = _reflection.GeneratedProtocolMessageType('ProofOps', (_message.Message,), {
  'DESCRIPTOR' : _PROOFOPS,
  '__module__' : 'tendermint.crypto.proof_pb2'
  # @@protoc_insertion_point(class_scope:tendermint.crypto.ProofOps)
  })
_sym_db.RegisterMessage(ProofOps)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z8github.com/tendermint/tendermint/proto/tendermint/crypto'
  _PROOFOPS.fields_by_name['ops']._options = None
  _PROOFOPS.fields_by_name['ops']._serialized_options = b'\310\336\037\000'
  _PROOF._serialized_start=74
  _PROOF._serialized_end=145
  _VALUEOP._serialized_start=147
  _VALUEOP._serialized_end=210
  _DOMINOOP._serialized_start=212
  _DOMINOOP._serialized_end=266
  _PROOFOP._serialized_start=268
  _PROOFOP._serialized_end=318
  _PROOFOPS._serialized_start=320
  _PROOFOPS._serialized_end=377
# @@protoc_insertion_point(module_scope)



from typing import List

from google.protobuf import any_pb2

from .composers import Dex, Perp
from .proto.cosmos.authz.v1beta1 import tx_pb2 as cosmos_authz_tx_pb
from .proto.cosmos.bank.v1beta1 import tx_pb2 as cosmos_bank_tx_pb
from .proto.cosmos.base.v1beta1 import coin_pb2 as cosmos_base_coin_pb
from .proto.cosmos.distribution.v1beta1 import tx_pb2 as tx_pb
from .proto.cosmos.staking.v1beta1 import tx_pb2 as cosmos_staking_tx_pb


class Composer:
    dex = Dex
    perp = Perp

    @staticmethod
    def coin(amount: float, denom: str):
        return cosmos_base_coin_pb.Coin(amount=str(amount), denom=denom)

    @staticmethod
    def msg_send(from_address: str, to_address: str, coins: List[cosmos_base_coin_pb.Coin]):
        return cosmos_bank_tx_pb.MsgSend(
            from_address=from_address,
            to_address=to_address,
            amount=coins,
        )

    @staticmethod
    def msg_exec(grantee: str, msgs: List):
        any_msgs: List[any_pb2.Any] = []
        for msg in msgs:
            any_msg = any_pb2.Any()
            any_msg.Pack(msg, type_url_prefix="")
            any_msgs.append(any_msg)

        return cosmos_authz_tx_pb.MsgExec(grantee=grantee, msgs=any_msgs)

    @staticmethod
    def msg_revoke(granter: str, grantee: str, msg_type: str):
        return cosmos_authz_tx_pb.MsgRevoke(granter=granter, grantee=grantee, msg_type_url=msg_type)

    @staticmethod
    def msg_delegate(delegator_address: str, validator_address: str, amount: float):
        return cosmos_staking_tx_pb.MsgDelegate(
            delegator_address=delegator_address,
            validator_address=validator_address,
            amount=Composer.coin(amount=amount, denom="inj"),
        )

    @staticmethod
    def msg_withdraw_delegator_reward(delegator_address: str, validator_address: str):

        return tx_pb.MsgWithdrawDelegatorReward(
            delegator_address=delegator_address, validator_address=validator_address
        )

    # # data field format: [request-msg-header][raw-byte-msg-response]
    # # you need to figure out this magic prefix number to trim request-msg-header off the data
    # # this method handles only exchange responses
    # @staticmethod
    # def MsgResponses(data, simulation=False):
    #     if not simulation:
    #         data = bytes.fromhex(data)
    #     header_map = {
    #         "/nibiru.exchange.v1beta1.MsgCreateSpotLimitOrder": injective_exchange_tx_pb.MsgCreateSpotLimitOrderResponse,
    #         "/nibiru.exchange.v1beta1.MsgCreateSpotMarketOrder": injective_exchange_tx_pb.MsgCreateSpotMarketOrderResponse,
    #         "/nibiru.exchange.v1beta1.MsgCreateDerivativeLimitOrder": injective_exchange_tx_pb.MsgCreateDerivativeLimitOrderResponse,
    #         "/nibiru.exchange.v1beta1.MsgCreateDerivativeMarketOrder": injective_exchange_tx_pb.MsgCreateDerivativeMarketOrderResponse,
    #         "/nibiru.exchange.v1beta1.MsgCancelSpotOrder": injective_exchange_tx_pb.MsgCancelSpotOrderResponse,
    #         "/nibiru.exchange.v1beta1.MsgCancelDerivativeOrder": injective_exchange_tx_pb.MsgCancelDerivativeOrderResponse,
    #         "/nibiru.exchange.v1beta1.MsgBatchCancelSpotOrders": injective_exchange_tx_pb.MsgBatchCancelSpotOrdersResponse,
    #         "/nibiru.exchange.v1beta1.MsgBatchCancelDerivativeOrders": injective_exchange_tx_pb.MsgBatchCancelDerivativeOrdersResponse,
    #         "/nibiru.exchange.v1beta1.MsgBatchCreateSpotLimitOrders": injective_exchange_tx_pb.MsgBatchCreateSpotLimitOrdersResponse,
    #         "/nibiru.exchange.v1beta1.MsgBatchCreateDerivativeLimitOrders": injective_exchange_tx_pb.MsgBatchCreateDerivativeLimitOrdersResponse,
    #         "/nibiru.exchange.v1beta1.MsgBatchUpdateOrders": injective_exchange_tx_pb.MsgBatchUpdateOrdersResponse,
    #         "/nibiru.exchange.v1beta1.MsgDeposit": injective_exchange_tx_pb.MsgDepositResponse,
    #         "/nibiru.exchange.v1beta1.MsgWithdraw": injective_exchange_tx_pb.MsgWithdrawResponse,
    #         "/nibiru.exchange.v1beta1.MsgSubaccountTransfer": injective_exchange_tx_pb.MsgSubaccountTransferResponse,
    #         "/nibiru.exchange.v1beta1.MsgLiquidatePosition": injective_exchange_tx_pb.MsgLiquidatePositionResponse,
    #         "/nibiru.exchange.v1beta1.MsgIncreasePositionMargin": injective_exchange_tx_pb.MsgIncreasePositionMarginResponse,
    #         "/nibiru.auction.v1beta1.MsgBid": injective_auction_tx_pb.MsgBidResponse,
    #         "/nibiru.exchange.v1beta1.MsgCreateBinaryOptionsLimitOrder": injective_exchange_tx_pb.MsgCreateBinaryOptionsLimitOrderResponse,
    #         "/nibiru.exchange.v1beta1.MsgCreateBinaryOptionsMarketOrder": injective_exchange_tx_pb.MsgCreateBinaryOptionsMarketOrderResponse,
    #         "/nibiru.exchange.v1beta1.MsgCancelBinaryOptionsOrder": injective_exchange_tx_pb.MsgCancelBinaryOptionsOrderResponse,
    #         "/nibiru.exchange.v1beta1.MsgAdminUpdateBinaryOptionsMarket": injective_exchange_tx_pb.MsgAdminUpdateBinaryOptionsMarketResponse,
    #         "/nibiru.exchange.v1beta1.MsgInstantBinaryOptionsMarketLaunch": injective_exchange_tx_pb.MsgInstantBinaryOptionsMarketLaunchResponse,
    #         "/cosmos.bank.v1beta1.MsgSend": cosmos_bank_tx_pb.MsgSendResponse,
    #         "/cosmos.authz.v1beta1.MsgGrant": cosmos_authz_tx_pb.MsgGrantResponse,
    #         "/cosmos.authz.v1beta1.MsgExec": cosmos_authz_tx_pb.MsgExecResponse,
    #         "/cosmos.authz.v1beta1.MsgRevoke": cosmos_authz_tx_pb.MsgRevokeResponse,
    #         "/nibiru.oracle.v1beta1.MsgRelayPriceFeedPrice": injective_oracle_tx_pb.MsgRelayPriceFeedPriceResponse,
    #         "/nibiru.oracle.v1beta1.MsgRelayProviderPrices": injective_oracle_tx_pb.MsgRelayProviderPrices
    #     }

    #     response = tx_response_pb.TxResponseData.FromString(data)
    #     msgs = []
    #     for msg in response.messages:
    #         msgs.append(header_map[msg.header].FromString(msg.data))

    #     return msgs

    # @staticmethod
    # def UnpackMsgExecResponse(msg_type, data):
    #     header_map = {
    #         "MsgCreateSpotLimitOrder": nibiru_exchange_tx_pb.MsgCreateSpotLimitOrderResponse,
    #         "MsgCreateSpotMarketOrder": nibiru_exchange_tx_pb.MsgCreateSpotMarketOrderResponse,
    #         "MsgCreateDerivativeLimitOrder": nibiru_exchange_tx_pb.MsgCreateDerivativeLimitOrderResponse,
    #         "MsgCreateDerivativeMarketOrder": nibiru_exchange_tx_pb.MsgCreateDerivativeMarketOrderResponse,
    #         "MsgCancelSpotOrder": nibiru_exchange_tx_pb.MsgCancelSpotOrderResponse,
    #         "MsgCancelDerivativeOrder": nibiru_exchange_tx_pb.MsgCancelDerivativeOrderResponse,
    #         "MsgBatchCancelSpotOrders": nibiru_exchange_tx_pb.MsgBatchCancelSpotOrdersResponse,
    #         "MsgBatchCancelDerivativeOrders": nibiru_exchange_tx_pb.MsgBatchCancelDerivativeOrdersResponse,
    #         "MsgBatchCreateSpotLimitOrders": nibiru_exchange_tx_pb.MsgBatchCreateSpotLimitOrdersResponse,
    #         "MsgBatchCreateDerivativeLimitOrders": nibiru_exchange_tx_pb.MsgBatchCreateDerivativeLimitOrdersResponse,
    #         "MsgBatchUpdateOrders": nibiru_exchange_tx_pb.MsgBatchUpdateOrdersResponse,
    #         "MsgDeposit": nibiru_exchange_tx_pb.MsgDepositResponse,
    #         "MsgWithdraw": nibiru_exchange_tx_pb.MsgWithdrawResponse,
    #         "MsgSubaccountTransfer": nibiru_exchange_tx_pb.MsgSubaccountTransferResponse,
    #         "MsgLiquidatePosition": nibiru_exchange_tx_pb.MsgLiquidatePositionResponse,
    #         "MsgIncreasePositionMargin": nibiru_exchange_tx_pb.MsgIncreasePositionMarginResponse,
    #         "MsgCreateBinaryOptionsLimitOrder": nibiru_exchange_tx_pb.MsgCreateBinaryOptionsLimitOrderResponse,
    #         "MsgCreateBinaryOptionsMarketOrder": nibiru_exchange_tx_pb.MsgCreateBinaryOptionsMarketOrderResponse,
    #         "MsgCancelBinaryOptionsOrder": nibiru_exchange_tx_pb.MsgCancelBinaryOptionsOrderResponse,
    #         "MsgAdminUpdateBinaryOptionsMarket": nibiru_exchange_tx_pb.MsgAdminUpdateBinaryOptionsMarketResponse,
    #         "MsgInstantBinaryOptionsMarketLaunch": nibiru_exchange_tx_pb.MsgInstantBinaryOptionsMarketLaunchResponse
    #     }

    #     return header_map[msg_type].FromString(bytes(data, "utf-8"))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/base/store/v1beta1/listening.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)cosmos/base/store/v1beta1/listening.proto\x12\x19\x63osmos.base.store.v1beta1\"L\n\x0bStoreKVPair\x12\x11\n\tstore_key\x18\x01 \x01(\t\x12\x0e\n\x06\x64\x65lete\x18\x02 \x01(\x08\x12\x0b\n\x03key\x18\x03 \x01(\x0c\x12\r\n\x05value\x18\x04 \x01(\x0c\x42*Z(github.com/cosmos/cosmos-sdk/store/typesb\x06proto3')



_STOREKVPAIR = DESCRIPTOR.message_types_by_name['StoreKVPair']
StoreKVPair = _reflection.GeneratedProtocolMessageType('StoreKVPair', (_message.Message,), {
  'DESCRIPTOR' : _STOREKVPAIR,
  '__module__' : 'cosmos.base.store.v1beta1.listening_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.base.store.v1beta1.StoreKVPair)
  })
_sym_db.RegisterMessage(StoreKVPair)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z(github.com/cosmos/cosmos-sdk/store/types'
  _STOREKVPAIR._serialized_start=72
  _STOREKVPAIR._serialized_end=148
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/feegrant/v1beta1/tx.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n cosmos/feegrant/v1beta1/tx.proto\x12\x17\x63osmos.feegrant.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\x1a\x19\x63osmos_proto/cosmos.proto\"q\n\x11MsgGrantAllowance\x12\x0f\n\x07granter\x18\x01 \x01(\t\x12\x0f\n\x07grantee\x18\x02 \x01(\t\x12:\n\tallowance\x18\x03 \x01(\x0b\x32\x14.google.protobuf.AnyB\x11\xca\xb4-\rFeeAllowanceI\"\x1b\n\x19MsgGrantAllowanceResponse\"6\n\x12MsgRevokeAllowance\x12\x0f\n\x07granter\x18\x01 \x01(\t\x12\x0f\n\x07grantee\x18\x02 \x01(\t\"\x1c\n\x1aMsgRevokeAllowanceResponse2\xec\x01\n\x03Msg\x12p\n\x0eGrantAllowance\x12*.cosmos.feegrant.v1beta1.MsgGrantAllowance\x1a\x32.cosmos.feegrant.v1beta1.MsgGrantAllowanceResponse\x12s\n\x0fRevokeAllowance\x12+.cosmos.feegrant.v1beta1.MsgRevokeAllowance\x1a\x33.cosmos.feegrant.v1beta1.MsgRevokeAllowanceResponseB)Z\'github.com/cosmos/cosmos-sdk/x/feegrantb\x06proto3')



_MSGGRANTALLOWANCE = DESCRIPTOR.message_types_by_name['MsgGrantAllowance']
_MSGGRANTALLOWANCERESPONSE = DESCRIPTOR.message_types_by_name['MsgGrantAllowanceResponse']
_MSGREVOKEALLOWANCE = DESCRIPTOR.message_types_by_name['MsgRevokeAllowance']
_MSGREVOKEALLOWANCERESPONSE = DESCRIPTOR.message_types_by_name['MsgRevokeAllowanceResponse']
MsgGrantAllowance = _reflection.GeneratedProtocolMessageType('MsgGrantAllowance', (_message.Message,), {
  'DESCRIPTOR' : _MSGGRANTALLOWANCE,
  '__module__' : 'cosmos.feegrant.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.feegrant.v1beta1.MsgGrantAllowance)
  })
_sym_db.RegisterMessage(MsgGrantAllowance)

MsgGrantAllowanceResponse = _reflection.GeneratedProtocolMessageType('MsgGrantAllowanceResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGGRANTALLOWANCERESPONSE,
  '__module__' : 'cosmos.feegrant.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.feegrant.v1beta1.MsgGrantAllowanceResponse)
  })
_sym_db.RegisterMessage(MsgGrantAllowanceResponse)

MsgRevokeAllowance = _reflection.GeneratedProtocolMessageType('MsgRevokeAllowance', (_message.Message,), {
  'DESCRIPTOR' : _MSGREVOKEALLOWANCE,
  '__module__' : 'cosmos.feegrant.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.feegrant.v1beta1.MsgRevokeAllowance)
  })
_sym_db.RegisterMessage(MsgRevokeAllowance)

MsgRevokeAllowanceResponse = _reflection.GeneratedProtocolMessageType('MsgRevokeAllowanceResponse', (_message.Message,), {
  'DESCRIPTOR' : _MSGREVOKEALLOWANCERESPONSE,
  '__module__' : 'cosmos.feegrant.v1beta1.tx_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.feegrant.v1beta1.MsgRevokeAllowanceResponse)
  })
_sym_db.RegisterMessage(MsgRevokeAllowanceResponse)

_MSG = DESCRIPTOR.services_by_name['Msg']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\'github.com/cosmos/cosmos-sdk/x/feegrant'
  _MSGGRANTALLOWANCE.fields_by_name['allowance']._options = None
  _MSGGRANTALLOWANCE.fields_by_name['allowance']._serialized_options = b'\312\264-\rFeeAllowanceI'
  _MSGGRANTALLOWANCE._serialized_start=137
  _MSGGRANTALLOWANCE._serialized_end=250
  _MSGGRANTALLOWANCERESPONSE._serialized_start=252
  _MSGGRANTALLOWANCERESPONSE._serialized_end=279
  _MSGREVOKEALLOWANCE._serialized_start=281
  _MSGREVOKEALLOWANCE._serialized_end=335
  _MSGREVOKEALLOWANCERESPONSE._serialized_start=337
  _MSGREVOKEALLOWANCERESPONSE._serialized_end=365
  _MSG._serialized_start=368
  _MSG._serialized_end=604
# @@protoc_insertion_point(module_scope)



import hashlib
import json
from typing import Tuple

import requests
from bech32 import bech32_decode, bech32_encode, convertbits
from bip32 import BIP32
from ecdsa import BadSignatureError, SECP256k1, SigningKey, VerifyingKey
from ecdsa.util import sigencode_string_canonize
from mnemonic import Mnemonic

from .exceptions import ConvertError, DecodeError
from .proto.cosmos.crypto.secp256k1.keys_pb2 import PubKey as PubKeyProto

BECH32_PUBKEY_ACC_PREFIX = "nibipub"
BECH32_PUBKEY_VAL_PREFIX = "nibivaloperpub"
BECH32_PUBKEY_CONS_PREFIX = "nibivalconspub"

BECH32_ADDR_ACC_PREFIX = "nibi"
BECH32_ADDR_VAL_PREFIX = "nibivaloper"
BECH32_ADDR_CONS_PREFIX = "nibivalcons"

DEFAULT_DERIVATION_PATH = "m/44'/118'/0'/0/0"


class PrivateKey:
    """
    Class for wrapping SigningKey that is used for signature creation and public key derivation.

    :ivar signing_key: the ecdsa SigningKey instance
    :vartype signing_key: ecdsa.SigningKey
    """

    def __init__(self, _error_do_not_use_init_directly=None) -> None:
        """Unsupported, please use from_mnemonic to initialize."""
        if not _error_do_not_use_init_directly:
            raise TypeError("Please use PrivateKey.from_mnemonic() to construct me")
        self.signing_key: SigningKey = None

    @classmethod
    def generate(cls, path=DEFAULT_DERIVATION_PATH) -> Tuple[str, "PrivateKey"]:
        """
        Generate new private key with random mnemonic phrase

        :param path: the HD path that follows the BIP32 standard

        :return: A tuple of mnemonic phrase and PrivateKey instance
        """
        phrase = Mnemonic(language="english").generate(strength=256)
        return (phrase, cls.from_mnemonic(phrase, path))

    @classmethod
    def from_mnemonic(cls, words: str, path=DEFAULT_DERIVATION_PATH) -> "PrivateKey":
        """
        Create a PrivateKey instance from a given mnemonic phrase and a HD derivation path.
        If path is not given, default to Band's HD prefix 494 and all other indexes being zeroes.

        :param words: the mnemonic phrase for recover private key
        :param path: the HD path that follows the BIP32 standard

        :return: Initialized PrivateKey object
        """
        seed = Mnemonic("english").to_seed(words)
        self = cls(_error_do_not_use_init_directly=True)
        self.signing_key = SigningKey.from_string(
            BIP32.from_seed(seed).get_privkey_from_path(path),
            curve=SECP256k1,
        )
        return self

    @classmethod
    def from_hex(cls, priv: str) -> "PrivateKey":
        self = cls(_error_do_not_use_init_directly=True)
        self.signing_key = SigningKey.from_string(bytes.fromhex(priv), curve=SECP256k1)
        return self

    def to_hex(self) -> str:
        """
        Return a hex representation of signing key.
        """
        return self.signing_key.to_string().hex()

    def to_public_key(self) -> "PublicKey":
        """
        Return the PublicKey associated with this private key.

        :return: a PublicKey that can be used to verify the signatures made with this PrivateKey
        """
        public_key = PublicKey(_error_do_not_use_init_directly=True)
        public_key.verify_key = self.signing_key.get_verifying_key()
        return public_key

    def sign(self, msg: bytes) -> bytes:
        """
        Sign the given message using the edcsa sign_deterministic function.

        :param msg: the message that will be hashed and signed

        :return: a signature of this private key over the given message
        """
        return self.signing_key.sign_deterministic(msg, hashfunc=hashlib.sha256, sigencode=sigencode_string_canonize)
        # return self.signing_key.sign_deterministic(msg, hashfunc=sha3.keccak_256, sigencode=sigencode_string_canonize)


class PublicKey:
    """
    Class for wrapping VerifyKey using for signature verification. Adding method to encode/decode
    to Bech32 format.

    :ivar verify_key: the ecdsa VerifyingKey instance
    :vartype verify_key: ecdsa.VerifyingKey
    """

    def __init__(self, _error_do_not_use_init_directly=None) -> None:
        """Unsupported, please do not construct it directly."""
        if not _error_do_not_use_init_directly:
            raise TypeError("Please use PublicKey's factory methods to construct me")
        self.verify_key: VerifyingKey = None

    @classmethod
    def _from_bech32(cls, bech: str, prefix: str) -> "PublicKey":
        hrp, bz = bech32_decode(bech)
        assert hrp == prefix, "Invalid bech32 prefix"
        if bz is None:
            raise DecodeError("Cannot decode bech32")
        bz = convertbits(bz, 5, 8, False)
        self = cls(_error_do_not_use_init_directly=True)
        self.verify_key = VerifyingKey.from_string(bytes(bz[5:]), curve=SECP256k1, hashfunc=hashlib.sha256)
        return self

    @classmethod
    def from_acc_bech32(cls, bech: str) -> "PublicKey":
        return cls._from_bech32(bech, BECH32_PUBKEY_ACC_PREFIX)

    @classmethod
    def from_val_bech32(cls, bech: str) -> "PublicKey":
        return cls._from_bech32(bech, BECH32_PUBKEY_VAL_PREFIX)

    @classmethod
    def from_cons_bech32(cls, bech: str) -> "PublicKey":
        return cls._from_bech32(bech, BECH32_PUBKEY_CONS_PREFIX)

    def to_hex(self) -> str:
        """
        Return a hex representation of verified key.
        """
        return self.verify_key.to_string("compressed").hex()

    def to_public_key_proto(self) -> PubKeyProto:
        return PubKeyProto(key=self.verify_key.to_string("compressed"))

    def _to_bech32(self, prefix: str) -> str:
        five_bit_r = convertbits(
            # Append prefix public key type follow amino spec.
            bytes.fromhex("eb5ae98721") + self.verify_key.to_string("compressed"),
            8,
            5,
        )
        assert five_bit_r is not None, "Unsuccessful bech32.convertbits call"
        return bech32_encode(prefix, five_bit_r)

    def to_acc_bech32(self) -> str:
        """Return bech32-encoded with account public key prefix"""
        return self._to_bech32(BECH32_PUBKEY_ACC_PREFIX)

    def to_val_bech32(self) -> str:
        """Return bech32-encoded with validator public key prefix"""
        return self._to_bech32(BECH32_PUBKEY_VAL_PREFIX)

    def to_cons_bech32(self) -> str:
        """Return bech32-encoded with validator consensus public key prefix"""
        return self._to_bech32(BECH32_PUBKEY_CONS_PREFIX)

    def to_address(self) -> "Address":
        """Return address instance from this public key"""
        # pubkey = self.verify_key.to_string("uncompressed")
        # k = sha3.keccak_256()
        # k.update(pubkey[1:])
        # addr = k.digest()[12:]
        # return Address(addr)

        pubkey = self.verify_key.to_string("compressed")
        s = hashlib.new("sha256", pubkey).digest()
        r = hashlib.new("ripemd160", s).digest()
        return Address(r)

    def verify(self, msg: bytes, sig: bytes) -> bool:
        """
        Verify a signature made from the given message.

        :param msg: data signed by the `signature`, will be hashed using sha256 function
        :param sig: encoding of the signature

        :raises BadSignatureError: if the signature is invalid or malformed
        :return: True if the verification was successful
        """
        try:
            return self.verify_key.verify(sig, msg, hashfunc=hashlib.sha256)
        except BadSignatureError:
            return False


class Address:
    def __init__(self, addr: bytes) -> None:
        self.addr = addr
        self.number = 0
        self.sequence = 0

    def __eq__(self, o: "Address") -> bool:
        return self.addr == o.addr

    @classmethod
    def _from_bech32(cls, bech: str, prefix: str) -> "Address":
        hrp, bz = bech32_decode(bech)
        assert hrp == prefix, "Invalid bech32 prefix"
        if bz is None:
            raise DecodeError("Cannot decode bech32")
        eight_bit_r = convertbits(bz, 5, 8, False)
        if eight_bit_r is None:
            raise ConvertError("Cannot convert to 8 bit")

        return cls(bytes(eight_bit_r))

    @classmethod
    def from_acc_bech32(cls, bech: str) -> "Address":
        """Create an address instance from a bech32-encoded account address"""
        return cls._from_bech32(bech, BECH32_ADDR_ACC_PREFIX)

    @classmethod
    def from_val_bech32(cls, bech: str) -> "Address":
        """Create an address instance from a bech32-encoded validator address"""
        return cls._from_bech32(bech, BECH32_ADDR_VAL_PREFIX)

    @classmethod
    def from_cons_bech32(cls, bech: str) -> "Address":
        """Create an address instance from a bech32-encoded consensus address"""
        return cls._from_bech32(bech, BECH32_ADDR_CONS_PREFIX)

    def _to_bech32(self, prefix: str) -> str:
        five_bit_r = convertbits(self.addr, 8, 5)
        assert five_bit_r is not None, "Unsuccessful bech32.convertbits call"
        return bech32_encode(prefix, five_bit_r)

    def to_acc_bech32(self) -> str:
        """Return a bech32-encoded account address"""
        return self._to_bech32(BECH32_ADDR_ACC_PREFIX)

    def to_val_bech32(self) -> str:
        """Return a bech32-encoded validator address"""
        return self._to_bech32(BECH32_ADDR_VAL_PREFIX)

    def to_cons_bech32(self) -> str:
        """Return a bech32-encoded with consensus address"""
        return self._to_bech32(BECH32_ADDR_CONS_PREFIX)

    def to_hex(self) -> str:
        """Return a hex representation of address"""
        return self.addr.hex()

    def get_subaccount_id(self, index: int) -> str:
        """Return a hex representation of address"""
        id = index.to_bytes(12, byteorder='big').hex()
        return '0x' + self.addr.hex() + id

    def init_num_seq(self, lcd_endpoint: str) -> "Address":
        response = requests.get(
            f"{lcd_endpoint}/cosmos/auth/v1beta1/accounts/{self.to_acc_bech32()}",
            headers={'Accept-Encoding': 'application/json'},
        )
        if response.status_code != 200:
            raise ValueError("HTTP response status", response.status_code)
        resp = json.loads(response.text)
        acc = resp['account']
        self.number = int(acc['account_number'])
        self.sequence = int(acc['sequence'])
        return self

    def decrease_sequence(self):
        """If a tx failed the sequence should not increase"""
        self.sequence -= 1

    def get_sequence(self):
        current_seq = self.sequence
        self.sequence += 1
        return current_seq

    def get_number(self):
        return self.number



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: cosmos/distribution/v1beta1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos.base.query.v1beta1 import pagination_pb2 as cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2
from cosmos.distribution.v1beta1 import distribution_pb2 as cosmos_dot_distribution_dot_v1beta1_dot_distribution__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'cosmos/distribution/v1beta1/query.proto\x12\x1b\x63osmos.distribution.v1beta1\x1a*cosmos/base/query/v1beta1/pagination.proto\x1a\x14gogoproto/gogo.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\x1a.cosmos/distribution/v1beta1/distribution.proto\"\x14\n\x12QueryParamsRequest\"P\n\x13QueryParamsResponse\x12\x39\n\x06params\x18\x01 \x01(\x0b\x32#.cosmos.distribution.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\"D\n\'QueryValidatorOutstandingRewardsRequest\x12\x19\n\x11validator_address\x18\x01 \x01(\t\"{\n(QueryValidatorOutstandingRewardsResponse\x12O\n\x07rewards\x18\x01 \x01(\x0b\x32\x38.cosmos.distribution.v1beta1.ValidatorOutstandingRewardsB\x04\xc8\xde\x1f\x00\"<\n\x1fQueryValidatorCommissionRequest\x12\x19\n\x11validator_address\x18\x01 \x01(\t\"y\n QueryValidatorCommissionResponse\x12U\n\ncommission\x18\x01 \x01(\x0b\x32;.cosmos.distribution.v1beta1.ValidatorAccumulatedCommissionB\x04\xc8\xde\x1f\x00\"\xaf\x01\n\x1cQueryValidatorSlashesRequest\x12\x19\n\x11validator_address\x18\x01 \x01(\t\x12\x17\n\x0fstarting_height\x18\x02 \x01(\x04\x12\x15\n\rending_height\x18\x03 \x01(\x04\x12:\n\npagination\x18\x04 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest:\x08\x88\xa0\x1f\x00\x98\xa0\x1f\x01\"\xa5\x01\n\x1dQueryValidatorSlashesResponse\x12G\n\x07slashes\x18\x01 \x03(\x0b\x32\x30.cosmos.distribution.v1beta1.ValidatorSlashEventB\x04\xc8\xde\x1f\x00\x12;\n\npagination\x18\x02 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse\"_\n\x1dQueryDelegationRewardsRequest\x12\x19\n\x11\x64\x65legator_address\x18\x01 \x01(\t\x12\x19\n\x11validator_address\x18\x02 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\x84\x01\n\x1eQueryDelegationRewardsResponse\x12\x62\n\x07rewards\x18\x01 \x03(\x0b\x32\x1c.cosmos.base.v1beta1.DecCoinB3\xc8\xde\x1f\x00\xaa\xdf\x1f+github.com/cosmos/cosmos-sdk/types.DecCoins\"I\n\"QueryDelegationTotalRewardsRequest\x12\x19\n\x11\x64\x65legator_address\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\xd6\x01\n#QueryDelegationTotalRewardsResponse\x12M\n\x07rewards\x18\x01 \x03(\x0b\x32\x36.cosmos.distribution.v1beta1.DelegationDelegatorRewardB\x04\xc8\xde\x1f\x00\x12`\n\x05total\x18\x02 \x03(\x0b\x32\x1c.cosmos.base.v1beta1.DecCoinB3\xc8\xde\x1f\x00\xaa\xdf\x1f+github.com/cosmos/cosmos-sdk/types.DecCoins\"F\n\x1fQueryDelegatorValidatorsRequest\x12\x19\n\x11\x64\x65legator_address\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"@\n QueryDelegatorValidatorsResponse\x12\x12\n\nvalidators\x18\x01 \x03(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"K\n$QueryDelegatorWithdrawAddressRequest\x12\x19\n\x11\x64\x65legator_address\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"K\n%QueryDelegatorWithdrawAddressResponse\x12\x18\n\x10withdraw_address\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"\x1b\n\x19QueryCommunityPoolRequest\"}\n\x1aQueryCommunityPoolResponse\x12_\n\x04pool\x18\x01 \x03(\x0b\x32\x1c.cosmos.base.v1beta1.DecCoinB3\xaa\xdf\x1f+github.com/cosmos/cosmos-sdk/types.DecCoins\xc8\xde\x1f\x00\x32\xd8\x0f\n\x05Query\x12\x98\x01\n\x06Params\x12/.cosmos.distribution.v1beta1.QueryParamsRequest\x1a\x30.cosmos.distribution.v1beta1.QueryParamsResponse\"+\x82\xd3\xe4\x93\x02%\x12#/cosmos/distribution/v1beta1/params\x12\x83\x02\n\x1bValidatorOutstandingRewards\x12\x44.cosmos.distribution.v1beta1.QueryValidatorOutstandingRewardsRequest\x1a\x45.cosmos.distribution.v1beta1.QueryValidatorOutstandingRewardsResponse\"W\x82\xd3\xe4\x93\x02Q\x12O/cosmos/distribution/v1beta1/validators/{validator_address}/outstanding_rewards\x12\xe2\x01\n\x13ValidatorCommission\x12<.cosmos.distribution.v1beta1.QueryValidatorCommissionRequest\x1a=.cosmos.distribution.v1beta1.QueryValidatorCommissionResponse\"N\x82\xd3\xe4\x93\x02H\x12\x46/cosmos/distribution/v1beta1/validators/{validator_address}/commission\x12\xd6\x01\n\x10ValidatorSlashes\x12\x39.cosmos.distribution.v1beta1.QueryValidatorSlashesRequest\x1a:.cosmos.distribution.v1beta1.QueryValidatorSlashesResponse\"K\x82\xd3\xe4\x93\x02\x45\x12\x43/cosmos/distribution/v1beta1/validators/{validator_address}/slashes\x12\xed\x01\n\x11\x44\x65legationRewards\x12:.cosmos.distribution.v1beta1.QueryDelegationRewardsRequest\x1a;.cosmos.distribution.v1beta1.QueryDelegationRewardsResponse\"_\x82\xd3\xe4\x93\x02Y\x12W/cosmos/distribution/v1beta1/delegators/{delegator_address}/rewards/{validator_address}\x12\xe8\x01\n\x16\x44\x65legationTotalRewards\x12?.cosmos.distribution.v1beta1.QueryDelegationTotalRewardsRequest\x1a@.cosmos.distribution.v1beta1.QueryDelegationTotalRewardsResponse\"K\x82\xd3\xe4\x93\x02\x45\x12\x43/cosmos/distribution/v1beta1/delegators/{delegator_address}/rewards\x12\xe2\x01\n\x13\x44\x65legatorValidators\x12<.cosmos.distribution.v1beta1.QueryDelegatorValidatorsRequest\x1a=.cosmos.distribution.v1beta1.QueryDelegatorValidatorsResponse\"N\x82\xd3\xe4\x93\x02H\x12\x46/cosmos/distribution/v1beta1/delegators/{delegator_address}/validators\x12\xf7\x01\n\x18\x44\x65legatorWithdrawAddress\x12\x41.cosmos.distribution.v1beta1.QueryDelegatorWithdrawAddressRequest\x1a\x42.cosmos.distribution.v1beta1.QueryDelegatorWithdrawAddressResponse\"T\x82\xd3\xe4\x93\x02N\x12L/cosmos/distribution/v1beta1/delegators/{delegator_address}/withdraw_address\x12\xb5\x01\n\rCommunityPool\x12\x36.cosmos.distribution.v1beta1.QueryCommunityPoolRequest\x1a\x37.cosmos.distribution.v1beta1.QueryCommunityPoolResponse\"3\x82\xd3\xe4\x93\x02-\x12+/cosmos/distribution/v1beta1/community_poolB3Z1github.com/cosmos/cosmos-sdk/x/distribution/typesb\x06proto3')



_QUERYPARAMSREQUEST = DESCRIPTOR.message_types_by_name['QueryParamsRequest']
_QUERYPARAMSRESPONSE = DESCRIPTOR.message_types_by_name['QueryParamsResponse']
_QUERYVALIDATOROUTSTANDINGREWARDSREQUEST = DESCRIPTOR.message_types_by_name['QueryValidatorOutstandingRewardsRequest']
_QUERYVALIDATOROUTSTANDINGREWARDSRESPONSE = DESCRIPTOR.message_types_by_name['QueryValidatorOutstandingRewardsResponse']
_QUERYVALIDATORCOMMISSIONREQUEST = DESCRIPTOR.message_types_by_name['QueryValidatorCommissionRequest']
_QUERYVALIDATORCOMMISSIONRESPONSE = DESCRIPTOR.message_types_by_name['QueryValidatorCommissionResponse']
_QUERYVALIDATORSLASHESREQUEST = DESCRIPTOR.message_types_by_name['QueryValidatorSlashesRequest']
_QUERYVALIDATORSLASHESRESPONSE = DESCRIPTOR.message_types_by_name['QueryValidatorSlashesResponse']
_QUERYDELEGATIONREWARDSREQUEST = DESCRIPTOR.message_types_by_name['QueryDelegationRewardsRequest']
_QUERYDELEGATIONREWARDSRESPONSE = DESCRIPTOR.message_types_by_name['QueryDelegationRewardsResponse']
_QUERYDELEGATIONTOTALREWARDSREQUEST = DESCRIPTOR.message_types_by_name['QueryDelegationTotalRewardsRequest']
_QUERYDELEGATIONTOTALREWARDSRESPONSE = DESCRIPTOR.message_types_by_name['QueryDelegationTotalRewardsResponse']
_QUERYDELEGATORVALIDATORSREQUEST = DESCRIPTOR.message_types_by_name['QueryDelegatorValidatorsRequest']
_QUERYDELEGATORVALIDATORSRESPONSE = DESCRIPTOR.message_types_by_name['QueryDelegatorValidatorsResponse']
_QUERYDELEGATORWITHDRAWADDRESSREQUEST = DESCRIPTOR.message_types_by_name['QueryDelegatorWithdrawAddressRequest']
_QUERYDELEGATORWITHDRAWADDRESSRESPONSE = DESCRIPTOR.message_types_by_name['QueryDelegatorWithdrawAddressResponse']
_QUERYCOMMUNITYPOOLREQUEST = DESCRIPTOR.message_types_by_name['QueryCommunityPoolRequest']
_QUERYCOMMUNITYPOOLRESPONSE = DESCRIPTOR.message_types_by_name['QueryCommunityPoolResponse']
QueryParamsRequest = _reflection.GeneratedProtocolMessageType('QueryParamsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPARAMSREQUEST,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryParamsRequest)
  })
_sym_db.RegisterMessage(QueryParamsRequest)

QueryParamsResponse = _reflection.GeneratedProtocolMessageType('QueryParamsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPARAMSRESPONSE,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryParamsResponse)
  })
_sym_db.RegisterMessage(QueryParamsResponse)

QueryValidatorOutstandingRewardsRequest = _reflection.GeneratedProtocolMessageType('QueryValidatorOutstandingRewardsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYVALIDATOROUTSTANDINGREWARDSREQUEST,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryValidatorOutstandingRewardsRequest)
  })
_sym_db.RegisterMessage(QueryValidatorOutstandingRewardsRequest)

QueryValidatorOutstandingRewardsResponse = _reflection.GeneratedProtocolMessageType('QueryValidatorOutstandingRewardsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYVALIDATOROUTSTANDINGREWARDSRESPONSE,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryValidatorOutstandingRewardsResponse)
  })
_sym_db.RegisterMessage(QueryValidatorOutstandingRewardsResponse)

QueryValidatorCommissionRequest = _reflection.GeneratedProtocolMessageType('QueryValidatorCommissionRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYVALIDATORCOMMISSIONREQUEST,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryValidatorCommissionRequest)
  })
_sym_db.RegisterMessage(QueryValidatorCommissionRequest)

QueryValidatorCommissionResponse = _reflection.GeneratedProtocolMessageType('QueryValidatorCommissionResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYVALIDATORCOMMISSIONRESPONSE,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryValidatorCommissionResponse)
  })
_sym_db.RegisterMessage(QueryValidatorCommissionResponse)

QueryValidatorSlashesRequest = _reflection.GeneratedProtocolMessageType('QueryValidatorSlashesRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYVALIDATORSLASHESREQUEST,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryValidatorSlashesRequest)
  })
_sym_db.RegisterMessage(QueryValidatorSlashesRequest)

QueryValidatorSlashesResponse = _reflection.GeneratedProtocolMessageType('QueryValidatorSlashesResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYVALIDATORSLASHESRESPONSE,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryValidatorSlashesResponse)
  })
_sym_db.RegisterMessage(QueryValidatorSlashesResponse)

QueryDelegationRewardsRequest = _reflection.GeneratedProtocolMessageType('QueryDelegationRewardsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDELEGATIONREWARDSREQUEST,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryDelegationRewardsRequest)
  })
_sym_db.RegisterMessage(QueryDelegationRewardsRequest)

QueryDelegationRewardsResponse = _reflection.GeneratedProtocolMessageType('QueryDelegationRewardsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDELEGATIONREWARDSRESPONSE,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryDelegationRewardsResponse)
  })
_sym_db.RegisterMessage(QueryDelegationRewardsResponse)

QueryDelegationTotalRewardsRequest = _reflection.GeneratedProtocolMessageType('QueryDelegationTotalRewardsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDELEGATIONTOTALREWARDSREQUEST,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryDelegationTotalRewardsRequest)
  })
_sym_db.RegisterMessage(QueryDelegationTotalRewardsRequest)

QueryDelegationTotalRewardsResponse = _reflection.GeneratedProtocolMessageType('QueryDelegationTotalRewardsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDELEGATIONTOTALREWARDSRESPONSE,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryDelegationTotalRewardsResponse)
  })
_sym_db.RegisterMessage(QueryDelegationTotalRewardsResponse)

QueryDelegatorValidatorsRequest = _reflection.GeneratedProtocolMessageType('QueryDelegatorValidatorsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDELEGATORVALIDATORSREQUEST,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryDelegatorValidatorsRequest)
  })
_sym_db.RegisterMessage(QueryDelegatorValidatorsRequest)

QueryDelegatorValidatorsResponse = _reflection.GeneratedProtocolMessageType('QueryDelegatorValidatorsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDELEGATORVALIDATORSRESPONSE,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryDelegatorValidatorsResponse)
  })
_sym_db.RegisterMessage(QueryDelegatorValidatorsResponse)

QueryDelegatorWithdrawAddressRequest = _reflection.GeneratedProtocolMessageType('QueryDelegatorWithdrawAddressRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDELEGATORWITHDRAWADDRESSREQUEST,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryDelegatorWithdrawAddressRequest)
  })
_sym_db.RegisterMessage(QueryDelegatorWithdrawAddressRequest)

QueryDelegatorWithdrawAddressResponse = _reflection.GeneratedProtocolMessageType('QueryDelegatorWithdrawAddressResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYDELEGATORWITHDRAWADDRESSRESPONSE,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryDelegatorWithdrawAddressResponse)
  })
_sym_db.RegisterMessage(QueryDelegatorWithdrawAddressResponse)

QueryCommunityPoolRequest = _reflection.GeneratedProtocolMessageType('QueryCommunityPoolRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYCOMMUNITYPOOLREQUEST,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryCommunityPoolRequest)
  })
_sym_db.RegisterMessage(QueryCommunityPoolRequest)

QueryCommunityPoolResponse = _reflection.GeneratedProtocolMessageType('QueryCommunityPoolResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYCOMMUNITYPOOLRESPONSE,
  '__module__' : 'cosmos.distribution.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.distribution.v1beta1.QueryCommunityPoolResponse)
  })
_sym_db.RegisterMessage(QueryCommunityPoolResponse)

_QUERY = DESCRIPTOR.services_by_name['Query']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z1github.com/cosmos/cosmos-sdk/x/distribution/types'
  _QUERYPARAMSRESPONSE.fields_by_name['params']._options = None
  _QUERYPARAMSRESPONSE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERYVALIDATOROUTSTANDINGREWARDSRESPONSE.fields_by_name['rewards']._options = None
  _QUERYVALIDATOROUTSTANDINGREWARDSRESPONSE.fields_by_name['rewards']._serialized_options = b'\310\336\037\000'
  _QUERYVALIDATORCOMMISSIONRESPONSE.fields_by_name['commission']._options = None
  _QUERYVALIDATORCOMMISSIONRESPONSE.fields_by_name['commission']._serialized_options = b'\310\336\037\000'
  _QUERYVALIDATORSLASHESREQUEST._options = None
  _QUERYVALIDATORSLASHESREQUEST._serialized_options = b'\210\240\037\000\230\240\037\001'
  _QUERYVALIDATORSLASHESRESPONSE.fields_by_name['slashes']._options = None
  _QUERYVALIDATORSLASHESRESPONSE.fields_by_name['slashes']._serialized_options = b'\310\336\037\000'
  _QUERYDELEGATIONREWARDSREQUEST._options = None
  _QUERYDELEGATIONREWARDSREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYDELEGATIONREWARDSRESPONSE.fields_by_name['rewards']._options = None
  _QUERYDELEGATIONREWARDSRESPONSE.fields_by_name['rewards']._serialized_options = b'\310\336\037\000\252\337\037+github.com/cosmos/cosmos-sdk/types.DecCoins'
  _QUERYDELEGATIONTOTALREWARDSREQUEST._options = None
  _QUERYDELEGATIONTOTALREWARDSREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYDELEGATIONTOTALREWARDSRESPONSE.fields_by_name['rewards']._options = None
  _QUERYDELEGATIONTOTALREWARDSRESPONSE.fields_by_name['rewards']._serialized_options = b'\310\336\037\000'
  _QUERYDELEGATIONTOTALREWARDSRESPONSE.fields_by_name['total']._options = None
  _QUERYDELEGATIONTOTALREWARDSRESPONSE.fields_by_name['total']._serialized_options = b'\310\336\037\000\252\337\037+github.com/cosmos/cosmos-sdk/types.DecCoins'
  _QUERYDELEGATORVALIDATORSREQUEST._options = None
  _QUERYDELEGATORVALIDATORSREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYDELEGATORVALIDATORSRESPONSE._options = None
  _QUERYDELEGATORVALIDATORSRESPONSE._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYDELEGATORWITHDRAWADDRESSREQUEST._options = None
  _QUERYDELEGATORWITHDRAWADDRESSREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYDELEGATORWITHDRAWADDRESSRESPONSE._options = None
  _QUERYDELEGATORWITHDRAWADDRESSRESPONSE._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYCOMMUNITYPOOLRESPONSE.fields_by_name['pool']._options = None
  _QUERYCOMMUNITYPOOLRESPONSE.fields_by_name['pool']._serialized_options = b'\252\337\037+github.com/cosmos/cosmos-sdk/types.DecCoins\310\336\037\000'
  _QUERY.methods_by_name['Params']._options = None
  _QUERY.methods_by_name['Params']._serialized_options = b'\202\323\344\223\002%\022#/cosmos/distribution/v1beta1/params'
  _QUERY.methods_by_name['ValidatorOutstandingRewards']._options = None
  _QUERY.methods_by_name['ValidatorOutstandingRewards']._serialized_options = b'\202\323\344\223\002Q\022O/cosmos/distribution/v1beta1/validators/{validator_address}/outstanding_rewards'
  _QUERY.methods_by_name['ValidatorCommission']._options = None
  _QUERY.methods_by_name['ValidatorCommission']._serialized_options = b'\202\323\344\223\002H\022F/cosmos/distribution/v1beta1/validators/{validator_address}/commission'
  _QUERY.methods_by_name['ValidatorSlashes']._options = None
  _QUERY.methods_by_name['ValidatorSlashes']._serialized_options = b'\202\323\344\223\002E\022C/cosmos/distribution/v1beta1/validators/{validator_address}/slashes'
  _QUERY.methods_by_name['DelegationRewards']._options = None
  _QUERY.methods_by_name['DelegationRewards']._serialized_options = b'\202\323\344\223\002Y\022W/cosmos/distribution/v1beta1/delegators/{delegator_address}/rewards/{validator_address}'
  _QUERY.methods_by_name['DelegationTotalRewards']._options = None
  _QUERY.methods_by_name['DelegationTotalRewards']._serialized_options = b'\202\323\344\223\002E\022C/cosmos/distribution/v1beta1/delegators/{delegator_address}/rewards'
  _QUERY.methods_by_name['DelegatorValidators']._options = None
  _QUERY.methods_by_name['DelegatorValidators']._serialized_options = b'\202\323\344\223\002H\022F/cosmos/distribution/v1beta1/delegators/{delegator_address}/validators'
  _QUERY.methods_by_name['DelegatorWithdrawAddress']._options = None
  _QUERY.methods_by_name['DelegatorWithdrawAddress']._serialized_options = b'\202\323\344\223\002N\022L/cosmos/distribution/v1beta1/delegators/{delegator_address}/withdraw_address'
  _QUERY.methods_by_name['CommunityPool']._options = None
  _QUERY.methods_by_name['CommunityPool']._serialized_options = b'\202\323\344\223\002-\022+/cosmos/distribution/v1beta1/community_pool'
  _QUERYPARAMSREQUEST._serialized_start=248
  _QUERYPARAMSREQUEST._serialized_end=268
  _QUERYPARAMSRESPONSE._serialized_start=270
  _QUERYPARAMSRESPONSE._serialized_end=350
  _QUERYVALIDATOROUTSTANDINGREWARDSREQUEST._serialized_start=352
  _QUERYVALIDATOROUTSTANDINGREWARDSREQUEST._serialized_end=420
  _QUERYVALIDATOROUTSTANDINGREWARDSRESPONSE._serialized_start=422
  _QUERYVALIDATOROUTSTANDINGREWARDSRESPONSE._serialized_end=545
  _QUERYVALIDATORCOMMISSIONREQUEST._serialized_start=547
  _QUERYVALIDATORCOMMISSIONREQUEST._serialized_end=607
  _QUERYVALIDATORCOMMISSIONRESPONSE._serialized_start=609
  _QUERYVALIDATORCOMMISSIONRESPONSE._serialized_end=730
  _QUERYVALIDATORSLASHESREQUEST._serialized_start=733
  _QUERYVALIDATORSLASHESREQUEST._serialized_end=908
  _QUERYVALIDATORSLASHESRESPONSE._serialized_start=911
  _QUERYVALIDATORSLASHESRESPONSE._serialized_end=1076
  _QUERYDELEGATIONREWARDSREQUEST._serialized_start=1078
  _QUERYDELEGATIONREWARDSREQUEST._serialized_end=1173
  _QUERYDELEGATIONREWARDSRESPONSE._serialized_start=1176
  _QUERYDELEGATIONREWARDSRESPONSE._serialized_end=1308
  _QUERYDELEGATIONTOTALREWARDSREQUEST._serialized_start=1310
  _QUERYDELEGATIONTOTALREWARDSREQUEST._serialized_end=1383
  _QUERYDELEGATIONTOTALREWARDSRESPONSE._serialized_start=1386
  _QUERYDELEGATIONTOTALREWARDSRESPONSE._serialized_end=1600
  _QUERYDELEGATORVALIDATORSREQUEST._serialized_start=1602
  _QUERYDELEGATORVALIDATORSREQUEST._serialized_end=1672
  _QUERYDELEGATORVALIDATORSRESPONSE._serialized_start=1674
  _QUERYDELEGATORVALIDATORSRESPONSE._serialized_end=1738
  _QUERYDELEGATORWITHDRAWADDRESSREQUEST._serialized_start=1740
  _QUERYDELEGATORWITHDRAWADDRESSREQUEST._serialized_end=1815
  _QUERYDELEGATORWITHDRAWADDRESSRESPONSE._serialized_start=1817
  _QUERYDELEGATORWITHDRAWADDRESSRESPONSE._serialized_end=1892
  _QUERYCOMMUNITYPOOLREQUEST._serialized_start=1894
  _QUERYCOMMUNITYPOOLREQUEST._serialized_end=1921
  _QUERYCOMMUNITYPOOLRESPONSE._serialized_start=1923
  _QUERYCOMMUNITYPOOLRESPONSE._serialized_end=2048
  _QUERY._serialized_start=2051
  _QUERY._serialized_end=4059
# @@protoc_insertion_point(module_scope)
